# -*- coding: utf-8 -*-
"""
Created on Wed Nov 22 23:26:20 2023

@author: clanglois1
"""
import os
from os.path import abspath

from inspect import getsourcefile
import numpy as np

from pyqtgraph.Qt import QtGui, QtCore
import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication

from inichord import general_functions as gf
from skimage.metrics import mean_squared_error as mse
from skimage.restoration import denoise_tv_chambolle
from skimage.metrics import structural_similarity as ssim

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/Denoise_v1.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self, parent):
        super().__init__()
        self.setupUi(self)
        self.parent = parent
        
        self.setWindowIcon(QtGui.QIcon('icons/filter_icon.png'))  
        
        self.expStack = parent.Current_stack
        
        self.Denoise_stack = np.copy(self.expStack)
        self.flag = parent.flag

        if self.flag == True:
            self.Reference = parent.Reference
            self.Slice1 = parent.Slice1
            self.displayRef(self.Reference)
            
            self.Proxy_button.setEnabled(False)
            
        else :
            self.Denoise_button.setEnabled(False)
            self.Denoise_button2.setEnabled(False)
            self.ChoiceBox.setEnabled(False)
            self.Choice_Idx.setEnabled(False)
            self.Spin_first.setEnabled(False)
            self.Spin_final.setEnabled(False)
            self.Spin_nbr.setEnabled(False)
            self.Validate_button.setEnabled(False)
            
        self.Proxy_button.clicked.connect(self.ImportProxyReference)
        
        self.Denoise_button.clicked.connect(self.AutoDenoisingStep)
        self.Denoise_button2.clicked.connect(self.StackDenoising)
        self.Validate_button.clicked.connect(self.ImportCurrentStack)
        self.mouseLock.setVisible(False)
        
        self.x = 0
        self.y = 0
    
        self.crosshair_v1= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h1 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.plotIt = self.profiles0.getPlotItem()
        self.plotIt.addLine(x = self.expSeries.currentIndex)
        
        self.proxy1 = pg.SignalProxy(self.expSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy4 = pg.SignalProxy(self.expSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)
        
        self.expSeries.timeLine.sigPositionChanged.connect(self.drawCHORDprofiles)
        
        self.flagden = 0
        
        self.defaultdrawCHORDprofiles()
        self.displayExpStack(self.expStack)
        self.defaultIV()
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.15))
        self.resize(int(geometry.width() * 0.7), int(geometry.height() * 0.7))
        self.screen = screen
        
#%% Functions               
    def Init_ProgressBar(self): # Define Progress bar depending on the step
        if self.flagsearch == 1:
            self.img_number = len(self.sigma_range)
            self.prgbar = 0 # Outil pour la bar de progression
            self.progressBar.setValue(self.prgbar)
            self.progressBar.setRange(0, self.img_number-1)
        elif self.flagsearch == 0:
            self.img_number = len(self.expStack)
            self.prgbar = 0 # Outil pour la bar de progression
            self.progressBar.setValue(self.prgbar)
            self.progressBar.setRange(0, self.img_number-1)
            
    def progression_bar(self): # Progress bar function
        self.prgbar = self.ValSlice
        self.progressBar.setValue(self.prgbar)

    def ImportProxyReference(self): # Step to create a proxy of image reference
        value = self.AVG_box.value()
        self.AVG_value = value
        
        self.AVG_slices()
        self.displayExpStack(self.expStack)
        self.label_13.setText("Image reference")
        self.displayRef(self.Reference)
        
        self.Spin_first.setEnabled(True)
        self.Spin_final.setEnabled(True)
        self.Spin_nbr.setEnabled(True)
        
        self.Denoise_button.setEnabled(True)
        self.ChoiceBox.setEnabled(True)
        
    def AVG_slices(self): # Creation of a proxy of reference using a given number of images averaged together
        self.Slice1 = self.expStack[0,:,:]
        self.Reference = np.nanmean(self.expStack[0:self.AVG_value,:,:],axis=0)

    def AutoDenoisingStep(self): # Step to determine the best denoising parameter
        self.flagsearch = 1
        
        Spin_first_value = self.Spin_first.value()
        Spin_final_value = self.Spin_final.value()
        Spin_nbr_value = self.Spin_nbr.value()
        
        self.sigma_range = np.round(np.linspace(Spin_first_value, Spin_final_value, Spin_nbr_value, endpoint = True),4)
        
        self.Step_choice = self.ChoiceBox.currentText()
        
        if self.Step_choice == 'NLMD':            
            self.NLMD_approach()
            self.Compute_MSE_SSIM()
            self.drawParamProfiles()
            
            self.denSSIM = gf.NonLocalMeanDenoising(self.Slice1, self.idx_SSIM)
            self.denMSE = gf.NonLocalMeanDenoising(self.Slice1, self.idx_MSE)
            
        elif self.Step_choice == 'BM3D':        
            self.BM3D_approach()
            self.Compute_MSE_SSIM()
            self.drawParamProfiles()
            
            self.denSSIM = gf.BM3D(self.Slice1, self.idx_SSIM, isFast = True)
            self.denMSE = gf.BM3D(self.Slice1, self.idx_MSE, isFast = True)
            
        elif self.Step_choice == 'VSNR':            
            self.VSNR_approach()
            self.Compute_MSE_SSIM()            
            self.drawParamProfiles()
            
            filter_SSIM = [{'name':'Dirac', 'noise_level': self.idx_SSIM }] 
            filter_MSE = [{'name':'Dirac', 'noise_level': self.idx_MSE }] 
            
            self.denSSIM = gf.VSNR_funct(self.Slice1, filter_SSIM)
            self.denMSE = gf.VSNR_funct(self.Slice1, filter_MSE)
            
        elif self.Step_choice == 'TV Chambolle':
            self.TV_chamb_approach()
            self.Compute_MSE_SSIM()              
            self.drawParamProfiles()
            
            self.denSSIM = denoise_tv_chambolle(self.Slice1, self.idx_SSIM)
            self.denMSE = denoise_tv_chambolle(self.Slice1, self.idx_MSE)
            
        self.idx_SSIM_info = '%.3f'%(self.idx_SSIM)
        self.idx_MSE_info = '%.3f'%(self.idx_MSE)
        
        self.SSIM_info.setText('Parameter from SSIM: ' + self.idx_SSIM_info)
        self.MSE_info.setText('Parameter from MSE: ' + self.idx_MSE_info)
        
        self.denMerge = np.concatenate((self.denSSIM,self.denMSE),0)
        self.label_13.setText("SSIM and MSE denoising")
        self.displayRef(self.denMerge)
        
        self.Denoise_button2.setEnabled(True)
        self.Choice_Idx.setEnabled(True)

    def check_type(self,data): # Check if the data has type uint8 or uint16 and modify it to float32
        datatype = data.dtype

        if datatype == "float64":
            data = gf.convertToUint8(data)
            self.data = data.astype(np.float32)
        if datatype == "uint16":
            data = gf.convertToUint8(data)
            self.data = data.astype(np.float32)
        elif datatype == "uint8":
            self.data = data.astype(np.float32)
        elif datatype == "float32":
            self.data = data
            
        return self.data

    def NLMD_approach(self): # Compute NLMD denoising for determination of best parameter
        self.Init_ProgressBar()
        self.Recap_denoise_stack = []
        
        self.Reference = self.check_type(self.Reference) # Convert data to float32 if needed
        self.Slice1 = self.check_type(self.Slice1) # Convert data to float32 if needed

        self.Reference2 = gf.NonLocalMeanDenoising(self.Reference, param_h = 0, fastAlgo = True, size = 5, distance = 6)
        
        for j in range(0,len(self.sigma_range)):
            denoised = gf.NonLocalMeanDenoising(self.Slice1, param_h = self.sigma_range[j], fastAlgo = True, size = 5, distance = 6)    
            self.Recap_denoise_stack.append(denoised)
            
            QApplication.processEvents() 
            self.ValSlice = j
            self.progression_bar()
            
    def BM3D_approach(self): # Compute BM3D denoising for determination of best parameter 
        self.Init_ProgressBar()

        self.Recap_denoise_stack = []
        self.Reference2 = gf.BM3D(self.Reference, 0, isFast = True)
        
        for j in range(0,len(self.sigma_range)):
            denoised = gf.BM3D(self.Slice1, self.sigma_range[j], isFast = True)   
            self.Recap_denoise_stack.append(denoised)
            
            QApplication.processEvents() 
            self.ValSlice = j
            self.progression_bar()

    def VSNR_approach(self): # Compute VSNR denoising for determination of best parameter 
        self.Init_ProgressBar()
        self.Recap_denoise_stack = []
        filter_ = [{'name':'Dirac', 'noise_level':0}] 
        self.Reference2 = gf.VSNR_funct(self.Reference,filter_)
        
        for j in range(0,len(self.sigma_range)):
            
            filter_ = [{'name':'Dirac', 'noise_level': self.sigma_range[j]}] 
            denoised = gf.VSNR_funct(self.Slice1,filter_)
                       
            self.Recap_denoise_stack.append(denoised)
            
            QApplication.processEvents() 
            self.ValSlice = j
            self.progression_bar()

    def TV_chamb_approach(self): # Compute TV Chambolle denoising for determination of best parameter 
        self.Init_ProgressBar()
        self.Recap_denoise_stack = []
        self.Reference2 = denoise_tv_chambolle(self.Reference, 1e-9)
    
        for j in range(0,len(self.sigma_range)):
            denoised = denoise_tv_chambolle(self.Slice1, self.sigma_range[j])
            self.Recap_denoise_stack.append(denoised)
            
            QApplication.processEvents() 
            self.ValSlice = j
            self.progression_bar()

    def Compute_MSE_SSIM(self): # Computation of MSE and SSIM value between images
        #  MSE Computation
        self.MSE = [mse(self.Reference2, img) for img in self.Recap_denoise_stack]
        self.min_MSE = np.nanmin(self.MSE)
        self.idx_MSE = self.sigma_range[np.nanargmin(self.MSE)]
        
        #  SSIM Computation
        self.SSIM = [ssim(self.Reference2, img, data_range=self.Reference2.max() - self.Reference2.min()) for img in self.Recap_denoise_stack]
        self.max_SSIM = np.nanmax(self.SSIM)
        self.idx_SSIM = self.sigma_range[np.nanargmax(self.SSIM)]

    def StackDenoising(self):
        self.flagsearch = 0
        
        self.Idx = self.Choice_Idx.currentText()
        
        if self.Idx == 'SSIM':
            self.IdxUsed = self.idx_SSIM
        elif self.Idx == 'MSE':
            self.IdxUsed = self.idx_MSE
        
        self.Step_choice = self.ChoiceBox.currentText()
        
        if self.Step_choice == 'NLMD':                
            self.Denoise_stack = np.copy(self.expStack)
            
            self.NLMD_denStack()
            self.displayExpStack(self.Denoise_stack)
            
        elif self.Step_choice == 'BM3D':                
            self.Denoise_stack = np.copy(self.expStack)
            
            self.BM3D_denStack()
            self.displayExpStack(self.Denoise_stack)

        elif self.Step_choice == 'VSNR':                
            self.Denoise_stack = np.copy(self.expStack)
            
            self.VNSR_denStack()
            self.displayExpStack(self.Denoise_stack)   

        elif self.Step_choice == 'TV Chambolle':              
            self.Denoise_stack = np.copy(self.expStack)
            
            self.TVcham_denStack()
            self.displayExpStack(self.Denoise_stack)  
            
        self.flagden = 1    
        self.drawCHORDprofiles()
        self.Validate_button.setEnabled(True)

    def NLMD_denStack(self): # Denoising using NLMD ; parameter is the denoising parameter value
        self.Init_ProgressBar()
        
        self.expStack = self.check_type(self.expStack) # Convert data to float32 if needed
              
        for i in range(0,len(self.expStack)):          
            self.Denoise_stack[i,:,:] = gf.NonLocalMeanDenoising(self.expStack[i,:,:], self.IdxUsed)

            QApplication.processEvents() 
            self.ValSlice = i
            self.progression_bar()
    
    def BM3D_denStack(self): # Denoising using BM3D ; parameter is the denoising parameter value
        self.Init_ProgressBar()
        for i in range(0,len(self.expStack)):
            self.Denoise_stack[i,:,:] = gf.BM3D(self.expStack[i,:,:], self.IdxUsed, isFast = True)
    
            QApplication.processEvents() 
            self.ValSlice = i
            self.progression_bar()
    
    def VNSR_denStack(self): # Denoising using VSNR (Dirac) ; parameter is the denoising parameter value
        self.Init_ProgressBar()
        filter_ = [{'name':'Dirac', 'noise_level': self.IdxUsed }] 
        
        for i in range(0,len(self.expStack)):
            self.Denoise_stack[i,:,:] = gf.VSNR_funct(self.expStack[i,:,:], filter_)

            QApplication.processEvents() 
            self.ValSlice = i
            self.progression_bar()
    
    def TVcham_denStack(self): # Denoising using TV Chambolle ; parameter is the denoising parameter value
        self.Init_ProgressBar()
        for i in range(0,len(self.expStack)):
            self.Denoise_stack[i,:,:] = denoise_tv_chambolle(self.expStack[i,:,:], self.IdxUsed)

            QApplication.processEvents() 
            self.ValSlice = i
            self.progression_bar()

    def ImportCurrentStack(self): # Extract denoised stack to the main gui
        if self.Idx == 'MSE':
            self.FlagIdx = self.idx_MSE
        elif self.Idx == 'SSIM':
            self.FlagIdx = self.idx_SSIM
        
        self.parent.Current_stack = self.Denoise_stack
        self.parent.StackList.append(self.Denoise_stack)
        
        Combo_text = '\u2022 Auto denoising using : ' + str(self.Step_choice) + '. Indice : ' + str(self.Idx) + '. Parameter : ' + str(self.FlagIdx)
        Combo_data = self.Denoise_stack
        self.parent.choiceBox.addItem(Combo_text, Combo_data)
        
        self.parent.displayExpStack(self.parent.Current_stack)
        
        self.parent.Info_box.ensureCursorVisible()
        self.parent.Info_box.insertPlainText("\n \u2022 Auto denoising is achieved")
        
        self.close()

    def defaultdrawCHORDprofiles(self):
        self.profiles.clear()
        self.profiles.setBackground(self.parent.color2)
        
        self.profiles.getPlotItem().hideAxis('bottom')
        self.profiles.getPlotItem().hideAxis('left')
        
        self.profiles0.clear()
        self.profiles0.setBackground(self.parent.color2)
        
        self.profiles0.getPlotItem().hideAxis('bottom')
        self.profiles0.getPlotItem().hideAxis('left')
        
        self.profiles2.clear()
        self.profiles2.setBackground(self.parent.color2)
        
        self.profiles2.getPlotItem().hideAxis('bottom')
        self.profiles2.getPlotItem().hideAxis('left')

    def drawParamProfiles(self):
        try:
            self.profiles.clear()
            self.profiles2.clear()
            
            pen = pg.mkPen(color=self.parent.color4, width=5) # Color and line width of the profile         

            self.profiles.plot(self.sigma_range,self.SSIM, pen=pen) # Plot of the SSIM Index
            self.profiles2.plot(self.sigma_range,self.MSE, pen=pen) # Plot of the MSE Index

            styles = {"color": "black", "font-size": "40px", "font-family": "Noto Sans Cond"} # Style for labels
            self.profiles.setLabel("left", "SSIM value", **styles) # Import style for Y label
            self.profiles.setLabel("bottom", "Denoising parameter", **styles) # Import style for X label
            
            self.profiles.getAxis('left').setTextPen('k') # Set the axis in black
            self.profiles.getAxis('bottom').setTextPen('k') # Set the axis in black
            self.profiles.setBackground(self.parent.color2)
            self.profiles.showGrid(x=True, y=True)

            self.profiles2.setLabel("left", "MSE value", **styles) # Import style for Y label
            self.profiles2.setLabel("bottom", "Denoising parameter", **styles) # Import style for X label
           
            self.profiles2.getAxis('left').setTextPen('k') # Set the axis in black
            self.profiles2.getAxis('bottom').setTextPen('k') # Set the axis in black 
            self.profiles2.setBackground(self.parent.color2)
            self.profiles2.showGrid(x=True, y=True)
            
        except:
            pass
        
    def drawCHORDprofiles(self):
        try:
            self.profiles0.clear()
            line = self.plotIt.addLine(x = self.expSeries.currentIndex)
            line.setPen({'color': (42, 42, 42, 100), 'width': 2})
            
            self.legend = self.profiles0.addLegend(horSpacing = 30, labelTextSize = '10pt', colCount = 1, labelTextColor = 'black', brush = self.parent.color6, pen = pg.mkPen(color=(0, 0, 0), width=1))
            
            pen = pg.mkPen(color=self.parent.color4, width=5) # Color and line width of the profile
            self.profiles0.plot(self.expStack[:, self.x, self.y], pen=pen, name='Undenoised') # Plot of the profile
            
            styles = {"color": "black", "font-size": "40px", "font-family": "Noto Sans Cond"} # Style for labels
            self.profiles0.setLabel("left", "Grayscale value", **styles) # Import style for Y label
            self.profiles0.setLabel("bottom", "Slice", **styles) # Import style for X label
            
            font=QtGui.QFont('Noto Sans Cond', 9)
            
            self.profiles0.getAxis("left").setTickFont(font) # Apply size of the ticks label
            self.profiles0.getAxis("left").setStyle(tickTextOffset = 20) # Apply a slight offset
            self.profiles0.getAxis("bottom").setTickFont(font) # Apply size of the ticks label
            self.profiles0.getAxis("bottom").setStyle(tickTextOffset = 20) # Apply a slight offset
            
            self.profiles0.getAxis('left').setTextPen('k') # Set the axis in black
            self.profiles0.getAxis('bottom').setTextPen('k') # Set the axis in black
            
            self.profiles0.setBackground(self.parent.color2)
            self.profiles0.showGrid(x=True, y=True)
            
            if self.flagden == 1:
                pen2 = pg.mkPen(color=self.parent.color5, width=5) # Color and line width of the profile
                self.profiles0.plot(self.Denoise_stack[:, self.x, self.y], pen=pen2, name='Denoised')
            
        except:
            pass

    def defaultIV(self):
        self.refSeries.ui.histogram.hide()
        self.refSeries.ui.roiBtn.hide()
        self.refSeries.ui.menuBtn.hide()
        
        view = self.refSeries.getView()
        view.setBackgroundColor(self.parent.color1)

    def displayExpStack(self, Series):
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        self.expSeries.addItem(self.crosshair_v1, ignoreBounds=True)
        self.expSeries.addItem(self.crosshair_h1, ignoreBounds=True) 
        
        view = self.expSeries.getView()
        state = view.getState()        
        self.expSeries.setImage(Series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.expSeries.timeLine.setPen(color=self.parent.color3, width=15)
        self.expSeries.frameTicks.setPen(color=self.parent.color1, width=5)
        self.expSeries.frameTicks.setYRange((0, 1))

        s = self.expSeries.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 
        
    def displayRef(self, Series):
        self.refSeries.ui.histogram.hide()
        self.refSeries.ui.roiBtn.hide()
        self.refSeries.ui.menuBtn.hide()
        
        view = self.refSeries.getView()
        state = view.getState()        
        self.refSeries.setImage(Series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.refSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        self.refSeries.timeLine.setPen(color=self.parent.color3, width=15)
        self.refSeries.frameTicks.setPen(color=self.parent.color1, width=5)
        self.refSeries.frameTicks.setYRange((0, 1))

        s = self.refSeries.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 
        
        self.refSeries.autoRange()

    def mouseMoved(self, e):
        pos = e[0]

        if not self.mouseLock.isChecked():
            if self.expSeries.view.sceneBoundingRect().contains(pos):
    
                item = self.expSeries.view
                mousePoint = item.mapSceneToView(pos) 
                     
                self.crosshair_v1.setPos(mousePoint.x())
                self.crosshair_h1.setPos(mousePoint.y())
    
            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
            if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0]) and self.y < len(self.expStack[0, 0, :]):
                self.drawCHORDprofiles()
    
    def mouseClick(self, e):
        pos = e[0]
        
        self.mouseLock.toggle()
        
        fromPosX = pos.scenePos()[0]
        fromPosY = pos.scenePos()[1]
        
        posQpoint = QtCore.QPointF()
        posQpoint.setX(fromPosX)
        posQpoint.setY(fromPosY)

        if self.expSeries.view.sceneBoundingRect().contains(posQpoint):
                
            item = self.expSeries.view
            mousePoint = item.mapSceneToView(posQpoint) 

            self.crosshair_v1.setPos(mousePoint.x())
            self.crosshair_h1.setPos(mousePoint.y())
                 
            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
        if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0])and self.y < len(self.expStack[0, 0, :]):
            self.drawCHORDprofiles()


# -*- coding: utf-8 -*-
"""
Created on Wed Nov 22 23:26:20 2023

@author: clanglois1
"""
import os
from os.path import abspath

from inspect import getsourcefile
import numpy as np

from pyqtgraph.Qt import QtGui, QtCore
import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication

from inichord import general_functions as gf

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/remOutliers_v4_TSG.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self, parent):
        super().__init__()
        self.setupUi(self)
        self.parent = parent
        
        self.setWindowIcon(QtGui.QIcon('icons/remove_outliers_icon.png'))
                
        self.expStack = parent.Current_stack
        self.denoised_Stack = np.copy(parent.Current_stack)
        
        self.x = 0
        self.y = 0
        
        self.radius = 2
        self.threshold = 10
        
        self.label_radius.setText("Radius: " + str(self.radius))
        self.label_threshold.setText("Threshold: " + str(self.threshold))
        
        self.img_number = len(self.expStack)
        
        if self.denoised.isChecked():
            self.denoised.toggle()
            
        self.denoised.setCheckable(False)

        self.crosshair_v1= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h1 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.plotIt = self.profiles.getPlotItem()
        self.plotIt.addLine(x = self.expSeries.currentIndex)
        
        self.proxy1 = pg.SignalProxy(self.expSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy4 = pg.SignalProxy(self.expSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)
        
        self.slider_threshold.valueChanged.connect(self.threshold_changed)
        self.slider_radius.valueChanged.connect(self.radius_changed)
        self.expSeries.timeLine.sigPositionChanged.connect(self.drawCHORDprofiles)
        self.preview.clicked.connect(self.remOutStack)
        self.denoised.stateChanged.connect(self.drawCHORDprofiles)
        
        self.Validate_button.clicked.connect(self.validate)
        self.mouseLock.setVisible(False)
        
        self.prgbar = 0 # Outil pour la bar de progression
        self.progressBar.setValue(self.prgbar)
        self.progressBar.setRange(0, self.img_number-1)
        
        self.remOutSlice()
        self.displayExpStack(self.denoised_Stack)
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.15))
        self.resize(int(geometry.width() * 0.7), int(geometry.height() * 0.6))
        self.screen = screen
        
        self.defaultdrawCHORDprofiles()
        self.Validate_button.setEnabled(False)

#%% Functions 
    def radius_changed(self):
        self.radius = self.slider_radius.value()
        self.label_radius.setText("Radius: " + str(self.radius))
        self.remOutSlice()
    
    def threshold_changed(self):
        self.threshold = self.slider_threshold.value()
        self.label_threshold.setText("Threshold: " + str(self.threshold))
        self.remOutSlice()
    
    def remOutSlice(self):
        dummy, a = gf.remove_outliers(self.expStack[0, :, :], self.radius, self.threshold)
        
        self.denoised_Stack[0, :, :] = a 
        self.displayExpStack(self.denoised_Stack)
        self.denoised.setEnabled(False)
        
    def remOutStack(self):
        self.denoised.setEnabled(False)
        self.preview.setEnabled(False)
        
        for i in range(0,len(self.expStack[:, 0, 0])): # Apply parameters on each slice
            _, self.denoised_Stack[i, :, :] =  gf.remove_outliers(self.expStack[i,:,:], self.radius, self.threshold)

            QApplication.processEvents()    
            self.ValSlice = i
            self.progression_bar()
       
        self.drawCHORDprofiles()                

        self.denoised.setEnabled(True)         
        self.denoised.setCheckable(True)
        self.denoised.setChecked(True)
        self.preview.setEnabled(True)
        self.Validate_button.setEnabled(True)
        
        self.displayExpStack(self.denoised_Stack)
        
    def validate(self):
        self.parent.Current_stack = np.copy(self.denoised_Stack)
        self.parent.StackList.append(self.denoised_Stack)
        
        Combo_text = '\u2022 Outliers filtering. Radius : ' + str(self.radius) + ' ; Threshold : ' + str(self.threshold)
        Combo_data = self.denoised_Stack
        self.parent.choiceBox.addItem(Combo_text, Combo_data)
        
        self.parent.displayExpStack(self.parent.Current_stack)
        
        self.parent.Info_box.ensureCursorVisible()
        self.parent.Info_box.insertPlainText("\n \u2022 Remove outliers is achieved.") 
        
        self.close()

    def drawCHORDprofiles(self):
        try:
            self.profiles.clear()
            line = self.plotIt.addLine(x = self.expSeries.currentIndex)
            line.setPen({'color': (42, 42, 42, 100), 'width': 2})
            
            self.legend = self.profiles.addLegend(horSpacing = 30, labelTextSize = '10pt', colCount = 1, labelTextColor = 'black', brush = self.parent.color6, pen = pg.mkPen(color=(0, 0, 0), width=1))
            
            pen = pg.mkPen(color=self.parent.color4, width=5) # Color and line width of the profile
            self.profiles.plot(self.expStack[:, self.x, self.y], pen=pen, name='Before remove') # Plot of the profile
            
            styles = {"color": "black", "font-size": "40px", "font-family": "Noto Sans Cond"} # Style for labels
            self.profiles.setLabel("left", "Grayscale value", **styles) # Import style for Y label
            self.profiles.setLabel("bottom", "Slice", **styles) # Import style for X label
            
            font=QtGui.QFont('Noto Sans Cond', 9)
            
            self.profiles.getAxis("left").setTickFont(font) # Apply size of the ticks label
            self.profiles.getAxis("left").setStyle(tickTextOffset = 20) # Apply a slight offset
            self.profiles.getAxis("bottom").setTickFont(font) # Apply size of the ticks label
            self.profiles.getAxis("bottom").setStyle(tickTextOffset = 20) # Apply a slight offset
            
            self.profiles.getAxis('left').setTextPen('k') # Set the axis in black
            self.profiles.getAxis('bottom').setTextPen('k') # Set the axis in black
            
            self.profiles.setBackground(self.parent.color2)
            self.profiles.showGrid(x=True, y=True)
            
            if self.denoised.isChecked():
                pen2 = pg.mkPen(color=self.parent.color5, width=5) # Color and line width of the profile

                self.profiles.plot(self.denoised_Stack[:, self.x, self.y], pen=pen2, name='After remove')
        except:
            pass
        
    def defaultdrawCHORDprofiles(self):
        self.profiles.clear()
        self.profiles.setBackground(self.parent.color2)
        
        self.profiles.getPlotItem().hideAxis('bottom')
        self.profiles.getPlotItem().hideAxis('left')

    def progression_bar(self): # Fonction relative à la barre de progression
        self.prgbar = self.ValSlice
        self.progressBar.setValue(self.prgbar)

    def displayExpStack(self, Series):
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        self.expSeries.addItem(self.crosshair_v1, ignoreBounds=True)
        self.expSeries.addItem(self.crosshair_h1, ignoreBounds=True) 
        
        view = self.expSeries.getView()
        state = view.getState()        
        self.expSeries.setImage(Series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.expSeries.timeLine.setPen(color=self.parent.color3, width=15)
        self.expSeries.frameTicks.setPen(color=self.parent.color1, width=5)
        self.expSeries.frameTicks.setYRange((0, 1))

        s = self.expSeries.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 

    def mouseMoved(self, e):
        pos = e[0]

        if not self.mouseLock.isChecked():
            if self.expSeries.view.sceneBoundingRect().contains(pos):
    
                item = self.expSeries.view
                mousePoint = item.mapSceneToView(pos) 
                     
                self.crosshair_v1.setPos(mousePoint.x())
                self.crosshair_h1.setPos(mousePoint.y())

            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
            if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0]) and self.y < len(self.expStack[0, 0, :]):
                self.drawCHORDprofiles()

    def mouseClick(self, e):
        pos = e[0]
        
        self.mouseLock.toggle()
        
        fromPosX = pos.scenePos()[0]
        fromPosY = pos.scenePos()[1]
        
        posQpoint = QtCore.QPointF()
        posQpoint.setX(fromPosX)
        posQpoint.setY(fromPosY)

        if self.expSeries.view.sceneBoundingRect().contains(posQpoint):
                
            item = self.expSeries.view
            mousePoint = item.mapSceneToView(posQpoint) 

            self.crosshair_v1.setPos(mousePoint.x())
            self.crosshair_h1.setPos(mousePoint.y())
                 
            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
        if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0])and self.y < len(self.expStack[0, 0, :]):
            self.drawCHORDprofiles()


# -*- coding: utf-8 -*-
"""
Created on Thu Sep  5 15:18:21 2024

@author: glhote1
"""

#%% Imports

import os
from os.path import abspath

from inspect import getsourcefile
import tifffile as tf
import numpy as np

import pyqtgraph as pg
from pyqtgraph.Qt import QtGui
from PyQt5.QtWidgets import QApplication

from inichord import general_functions as gf # USED
from inichord import Alignement_GUI_pg_12_TSG as align # USED
from inichord import remFFT_pg_v2_TSG as RemFFT # USED
from inichord import remOutliersGUI_pg_v3_TSG as rO # USED
from inichord import Denoise_GUI_pg_03_TSG as autoden # USED
from inichord import KAD_function_02 as KADfunc # USED
from inichord import Contour_map_pg_01_TSG as Contour

import tkinter as tk
from tkinter import filedialog

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/Batch_processing_ui_02.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self,parent):
        super().__init__()

        self.setupUi(self)
        self.setWindowIcon(QtGui.QIcon('icons/Main_icon.png')) # Application of the main icon
        self.parent = parent
        
        self.OpenData.clicked.connect(self.loaddata) # Load data
        self.Ref_bttn.clicked.connect(self.loadref) # Load image reference
        self.Run_bttn.clicked.connect(self.run_batch) # Run batch
        self.ChoiceBox_algo.currentTextChanged.connect(self.Filter_changed) # Change searching range
        
        self.flagref = 0
        
        self.Run_bttn.setEnabled(False) # Run button is disable until data opening.
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        # Position (self.move) and size (self.resize) of the main GUI on the screen
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.1))
        self.resize(int(geometry.width() * 0.4), int(geometry.height() * 0.6))
        self.screen = screen

    def loaddata(self):
        StackLoc, StackDir = gf.getFilePathDialog("Series") # Image importation
        
        self.image = [] # Initialization of the variable self.image (which will be the full series folder)
        
        if len(StackLoc) > 1:
            self.progressBar.setValue(0) # Set the initial value of the Progress bar at 0
            self.progressBar.setRange(0, len(StackLoc)-1) 
            self.progressBar.setFormat("Loading series... %p%")

        for i in range(0,len(StackLoc)):
            Var = tf.TiffFile(StackLoc[i]).asarray() # Import the unit 2D array
            self.image.append(Var) # Append every 2D array in a list named self.image
            
            QApplication.processEvents()    
            self.ValSlice = i
            self.progression_bar()
            
        self.progressBar.setFormat("Series have been loaded!")
        
        self.Info_box.ensureCursorVisible()
        self.Info_box.insertPlainText("\n Image series have been loaded.")
        self.Info_box.insertPlainText("\n ----------")
        
        self.Run_bttn.setEnabled(True)

    def loadref(self):
        StackLoc, StackDir = gf.getFilePathDialog("image reference") # Image importation
        
        self.ref = [] # Initialization of the variable self.image (which will be the full series folder)
        
        for i in range(0,len(StackLoc)):
            Var = tf.TiffFile(StackLoc[i]).asarray() # Import the unit 2D array
            self.ref.append(Var) # Append every 2D array in a list named self.image
        
        self.Info_box.ensureCursorVisible()
        self.Info_box.insertPlainText("\n Reference images have been loaded.")
        
        for i in range(0,len(self.image)):
            ref2 = np.reshape(self.ref[i],(-1,len(self.ref[i]),len(self.ref[i][0])))
            self.image[i] = np.concatenate((ref2, self.image[i]))
        
        self.flagref = 1
        self.meanRef_val.setEnabled(False)

    def Filter_changed(self): # KAD filtering processes
        self.Filter_choice = self.ChoiceBox_algo.currentText()
        
        if self.Filter_choice == "NLMD":
            self.spinStart = self.spinStart_val.setValue(0)
            self.spinEnd = self.spinEnd_val.setValue(20)
            self.spinNbr = self.spinNbr_val.setValue(30)
            
        elif self.Filter_choice == "BM3D":
            self.spinStart = self.spinStart_val.setValue(0)
            self.spinEnd = self.spinEnd_val.setValue(20)
            self.spinNbr = self.spinNbr_val.setValue(15)
            
        elif self.Filter_choice == "VSNR":
            self.spinStart = self.spinStart_val.setValue(0)
            self.spinEnd = self.spinEnd_val.setValue(5)
            self.spinNbr = self.spinNbr_val.setValue(15)
            
        elif self.Filter_choice == "TV Chambolle":
            self.spinStart = self.spinStart_val.setValue(0)
            self.spinEnd = self.spinEnd_val.setValue(20)
            self.spinNbr = self.spinNbr_val.setValue(30)

    def run_batch(self):
        # Initialiser Tkinter
        root = tk.Tk()
        root.withdraw()  # Masquer la fenêtre principale Tkinter
        # Ouvrir une boîte de dialogue pour choisir un dossier de sauvegarde
        dossier = filedialog.askdirectory(title="Choose a saving folder for treated series")
        
        if len(self.image) > 1:
            self.progressBar.setValue(0) # Set the initial value of the Progress bar at 0
            self.progressBar.setRange(0, len(self.image)-1) 
            self.progressBar.setFormat("Treatment... %p%")
        
        # Variable extraction      
        self.radius_reg = self.Radius_box.value()
        self.threshold_reg = self.Threshold_box.value()
        self.blur_reg = int(self.Blur_box.currentText())
        self.sobel_reg = int(self.Sobel_box.currentText())
        self.reg1 = self.transfo_comboBox.currentText()
        self.reg2 = self.transfo_comboBox2.currentText()
        
        self.FFT = self.FFT_val.value()
        
        self.radius_remout = self.radius_remout_val.value()
        self.threshold_remout = self.threshold_remout_val.value()
        
        self.spinStart = self.spinStart_val.value()
        self.spinEnd = self.spinEnd_val.value()
        self.spinNbr = self.spinNbr_val.value()
        self.meanRef = self.meanRef_val.value()
        
        self.radius = self.Radius_box_2.value()
        self.threshold = self.Threshold_box_2.value()
        self.blur = int(self.Blur_box_2.currentText())
        self.sobel = int(self.Sobel_box_2.currentText())
        
        self.Denoising_algo = self.ChoiceBox_algo.currentText()
        self.Index_algo = self.Choice_Idx.currentText()
                
        for i in range(0,len(self.image)):
            
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText(f"\n Current processed series: {i+1} out of {len(self.image)}")
            
            QApplication.processEvents()    
            self.ValSlice = i
            self.progression_bar()

            # Creation of the Main_TSG object that will store modifications
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Main object initialization.")
            
            Main_TSG = self.parent
            Main_TSG.Current_stack = self.image[i]
            
            # First registration step
            w = align.MainWindow(Main_TSG)
        
            w.radius_Val = self.radius_reg
            w.threshold_Val = self.threshold_reg
            w.blur_value = self.blur_reg
            w.sobel_value = self.sobel_reg
            w.choice_transfo = str(self.reg1)
        
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Registration step I in progress.")
            
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Microstructural features definition I.")
            w.Pre_treatment()

            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Start of sequential registration I.")
            w.Seq_registration()
            
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Crop extra black border I.")
            w.Crop_data()
            
            Main_TSG.Current_stack = w.Cropped_stack
            
            # Second registration step
            if self.reg2 != "None":
                
                self.Info_box.ensureCursorVisible()
                self.Info_box.insertPlainText("\n     Registration step II in progress.")
                
                wbis = align.MainWindow(Main_TSG)
            
                wbis.radius_Val = self.radius_reg
                wbis.threshold_Val = self.threshold_reg
                wbis.blur_value = self.blur_reg
                wbis.sobel_value = self.sobel_reg
                wbis.choice_transfo = str(self.reg2)
                
                self.Info_box.ensureCursorVisible()
                self.Info_box.insertPlainText("\n     Microstructural features definition II.")
                wbis.Pre_treatment()
  
                self.Info_box.ensureCursorVisible()
                self.Info_box.insertPlainText("\n     Start of sequential registration II.")
                wbis.Seq_registration()
                
                self.Info_box.ensureCursorVisible()
                self.Info_box.insertPlainText("\n     Crop extra black border II.")
                wbis.Crop_data()
            
                Main_TSG.Current_stack = wbis.Cropped_stack
 
            # Background substraction
            w2 = RemFFT.MainWindow(Main_TSG)
        
            w2.fft = self.FFT
            
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Background substraction step in progress.")
            w2.FFTStack()
        
            Main_TSG.Current_stack = w2.filtered_Stack
        
            # Outlier filtering            
            w3 = rO.MainWindow(Main_TSG)
        
            w3.radius = self.radius_remout
            w3.threshold = self.threshold_remout
            
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Remove outliers step in progress.")
            w3.remOutStack()
        
            Main_TSG.Current_stack = w3.denoised_Stack
        
            # Auto-denoising
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Denoising step in progress.")
            
            Main_TSG.flag = False
            
            w4 = autoden.MainWindow(Main_TSG)
        
            w4.AVG_value = self.meanRef
            w4.Spin_first_value = self.spinStart
            w4.Spin_final_value = self.spinEnd
            w4.Spin_nbr_value = self.spinNbr
            w4.Step_choice = self.Denoising_algo
            w4.Idx = self.Index_algo
            
            if self.flagref == 0 :
                self.Info_box.ensureCursorVisible()
                self.Info_box.insertPlainText("\n     Proxy image of reference creation.")
                w4.AVG_slices()
                
            elif self.flagref == 1 :
                self.Info_box.ensureCursorVisible()
                self.Info_box.insertPlainText("\n     Search using the image reference.")
                
                w4.Reference = np.copy(w4.expStack[0,:,:]) # Extraction of the image of reference
                w4.Slice1 = np.copy(w4.expStack[1,:,:]) # Copy of the first slice of the stack
                w4.expStack = np.copy(w4.expStack[1:,:,:]) # Create of the Current stack without reference
                w4.Denoise_stack = np.copy(w4.expStack)
                
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Search for the perfect denoising parameter.")
            w4.AutoDenoisingStep()
            
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     The SSIM denoising parameters is : " + str(w4.idx_SSIM_info))
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     The MSE denoising parameters is : " + str(w4.idx_MSE_info))
            
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Stack denoising using the absolute and perfect denoising parameter.")
            w4.StackDenoising()
        
            Main_TSG.Current_stack = w4.Denoise_stack
        
            # KAD computation 
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     KAD computation step in progress.")
            
            stack_norm = KADfunc.centeredEuclidianNorm(Main_TSG.Current_stack, 0) # Normalization of the image series
            KAD = KADfunc.Divided_KAD(stack_norm) # Compute the KAD map
            
            # Contour map computation
            self.Info_box.ensureCursorVisible()
            self.Info_box.insertPlainText("\n     Contour map in progress.")
            
            w5 = Contour.MainWindow(Main_TSG)
            
            w5.radius = self.radius
            w5.threshold = self.threshold
            w5.blur = self.blur
            w5.sobel = self.sobel
            
            w5.compute_map()
            Contour_map = w5.contour_map
            
            # ask for saving datas 
            chemin_complet = os.path.join(dossier, f"Treated_serie_{i+1}.tiff")
            # Sauvegarder le tableau 3D au format TIFF avec le type de données d'origine
            tf.imwrite(chemin_complet, Main_TSG.Current_stack)
            
            chemin_complet = os.path.join(dossier, f"KAD_{i+1}.tiff")
            # Sauvegarder le tableau 3D au format TIFF avec le type de données d'origine
            tf.imwrite(chemin_complet, KAD)
            
            chemin_complet = os.path.join(dossier, f"Contour_{i+1}.tiff")
            # Sauvegarder le tableau 3D au format TIFF avec le type de données d'origine
            tf.imwrite(chemin_complet, Contour_map)
            
            del(KAD, stack_norm, Main_TSG, Contour_map)
            
        self.Info_box.ensureCursorVisible()
        self.Info_box.insertPlainText("\n ----------")
        self.Info_box.insertPlainText("\n Treated series have been saved.")

    def progression_bar(self): # Function for the ProgressBar uses
        self.prgbar = self.ValSlice
        self.progressBar.setValue(self.prgbar)





import os
from os.path import abspath

from inspect import getsourcefile
import numpy as np
import tifffile as tf
import cv2
import largestinteriorrectangle as lir
import scipy.ndimage as sci
from pystackreg import StackReg

from pyqtgraph.Qt import QtGui
import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication, QMessageBox

from inichord import general_functions as gf

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/Aligned_pg_ui_12_TSG.ui")

class MainWindow(uiclass, baseclass):

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.setupUi(self)
        
        self.setWindowIcon(QtGui.QIcon('icons/alignment_icon.png'))
        
        self.expStack = parent.Current_stack
        self.Pre_treatment_stack = np.copy(parent.Current_stack) # Stack that will be used to process the pretreatment 
        
        # Stack that will store the final result
        self.Aligned_stack = np.copy(parent.Current_stack)
        
        self.type = self.expStack.dtype

        if self.type == "uint16" or self.type == "uint32" or self.type == "float64" or self.type == "float32":
            self.expStack = gf.convertToUint8(self.expStack) # Necessary for computation
            self.Pre_treatment_stack = gf.convertToUint8(self.Pre_treatment_stack)
            self.Aligned_stack = gf.convertToUint8(self.Pre_treatment_stack)

        self.Pre_treatment_stack_Remout = np.copy(self.Pre_treatment_stack)
        self.Pre_treatment_stack_output = np.copy(self.Pre_treatment_stack)
        self.Pre_treatment_stack_output2 = np.copy(self.Pre_treatment_stack)

        self.Pre_treatment_doppelstack = np.copy(self.Pre_treatment_stack)
        self.Pre_treatment_doppelstack_Remout = np.copy(self.Pre_treatment_stack)
        self.Pre_treatment_doppelstack_output = np.copy(self.Pre_treatment_stack)
        self.Pre_treatment_doppelstack_output2 = np.copy(self.Pre_treatment_stack)

        self.blur_value = int(self.Blur_box.currentText())
        self.sobel_value = int(self.Sobel_box.currentText())

        self.im_reference = 0 # n° of the reference image (for sequential approach)
        self.iter_value = 2000 # Number of iteration to converged
        self.thres_value = 0.000001 # Convergence value 
        
        self.img_number = len(self.expStack)
        self.stack_height = len(self.expStack[0])
        self.stack_width = len(self.expStack[0][0])
        
        self.radius_Val = self.Radius_slider.value() # Input filtre Remout
        self.threshold_Val = self.Threshold_slider.value()  # Input filtre Remout
        
        self.Label_radius.setText("Remove Outliers - Radius: " + str(self.radius_Val)) # Input filtre Remout
        self.Label_threshold.setText("Remove Outliers - Threshold: " + str(self.threshold_Val)) # Input filtre Remout
        
        self.Radius_slider.valueChanged.connect(self.radius_changed) # Input filtre Remout
        self.Threshold_slider.valueChanged.connect(self.threshold_changed) # Input filtre Remout
        
        self.Blur_box.activated.connect(self.get_blur_changed) # Gaussian blur changes connexion
        self.Sobel_box.activated.connect(self.get_sobel_changed) # Sobel changes connexion
        
        self.push_compute.clicked.connect(self.Choice_method)
        self.Push_valid.clicked.connect(self.extract_align_stack) # Extract data to the main gui
        self.Push_valid.setEnabled(False) # Validation Unauthorized at the beginning 
        
        self.progressBar.setValue(0) # Progress bar initial value is set to 0
        self.progressBar.setRange(0, self.img_number-1)
        
        self.checkBox.clicked.connect(self.checking) # Check if the registration must be done on the raw stack or on the pretreated one
        self.checkBox_Uncropped.clicked.connect(self.checking_Uncrop) # Check if the final result is cropped or no
        self.Keep_param.clicked.connect(self.Pre_treatment) # apply pretreatment parameter to all slices
        
        if self.checkBox_Uncropped.isChecked() == False: 
            self.flagCrop = "No"
        
        self.Pre_treatment_slice() # Run a pretreatment on 1/10 images
        
        self.transfo_comboBox.setCurrentIndex(4) 
        self.wrapmode  = cv2.MOTION_HOMOGRAPHY
        self.textEdit.insertPlainText("\n Homographic transformation used to estim correlation.")
        self.flagTransformation = "Homography"
         
        self.CC_info.setText('Estimated correlation coefficient.') # Information about the correlation coefficient between 2 images (pretreated)
        self.textEdit.setFontPointSize(10)
        
        self.Coefficient_estim() # Estim corrcoeff between the 2 first images (Pretreated)
        
        self.defaultdrawCC()
        self.displayExpStack(self.expStack)
        self.display_Aligne_treatment(self.Aligned_stack)
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        # Position (self.move) and size (self.resize) of the main GUI on the screen
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.15))
        self.resize(int(geometry.width() * 0.8), int(geometry.height() * 0.7))
        self.screen = screen

#%% Functions : initialization
    def checking(self): # Allow to define if registration must be applied to the raw stack or to the pretreated one
        if self.checkBox.isChecked() == True:
            self.checkBox.clicked.connect(self.Pre_notreatment) # Registration on the raw stack
            self.textEdit.insertPlainText("\n Registration performs on raw images.")
        elif self.checkBox.isChecked() == False:
            self.Blur_box.activated.connect(self.radius_changed) 
            self.Sobel_box.activated.connect(self.threshold_changed)
            self.Blur_box.activated.connect(self.get_blur_changed) 
            self.Sobel_box.activated.connect(self.get_sobel_changed)
            self.textEdit.insertPlainText("\n Registration performs using the treated stack.")

    def checking_Uncrop(self): # Define if the final stack must be cropped
        if self.checkBox_Uncropped.isChecked() == True:
            self.textEdit.insertPlainText("\n Uncropped stack will be saved.")
            self.flagCrop = "Yes" 
        elif self.checkBox_Uncropped.isChecked() == False:
            self.textEdit.insertPlainText("\n Cropped stack will be saved.")
            self.flagCrop = "No"

    def warp_mode_selection(self): # Define the wrapmode dimensions depending if an affine or an homographic trasnformation is used
        if self.choice_transfo == "Affine":
            self.wrapmode  = cv2.MOTION_AFFINE
            self.flagTransformation = "Affine"
        elif self.choice_transfo == "Homography":
            self.wrapmode  = cv2.MOTION_HOMOGRAPHY
            self.flagTransformation = "Homography"
        
        return self.wrapmode
               
    def radius_changed(self): # apply remove outliers radius modification
        value = self.Radius_slider.value()
        self.radius_Val = value
        self.Label_radius.setText("Remove Outliers - Radius: " + str(self.radius_Val)) 
        self.Coefficient_estim()
        self.Pre_treatment_slice()

    def threshold_changed(self):  # Apply remove outliers threshold modification
        value = self.Threshold_slider.value()
        self.threshold_Val = value
        self.Label_threshold.setText("Remove Outliers - Threshold: " + str(self.threshold_Val))
        self.Coefficient_estim()
        self.Pre_treatment_slice()  
        
    def get_blur_changed(self):  # Apply Gaussian blur modification
        value = self.Blur_box.currentText()
        self.blur_value = int(value)
        self.Coefficient_estim()
        self.Pre_treatment_slice()

    def get_sobel_changed(self): # Apply Sobel factor modification
        value = self.Sobel_box.currentText()
        self.sobel_value = int(value)
        self.Coefficient_estim()
        self.Pre_treatment_slice()

    def defaultdrawCC(self): # Default display of the coefficient correlation
        self.RecapCC.clear()
        self.RecapCC.setBackground(self.parent.color2)
        
        self.RecapCC.getPlotItem().hideAxis('bottom')
        self.RecapCC.getPlotItem().hideAxis('left')

    def drawRecapCC(self,recap): # Plot Corrcoeff during registration (affine and homography only)
        self.RecapCC.clear()
                  
        pen = pg.mkPen(color=self.parent.color4, width=5) # Color and line width of the profile
        self.RecapCC.plot(recap[:, 0], pen=pen) # Plot of the profile
        
        styles = {"color": "black", "font-size": "40px", "font-family": "Noto Sans Cond"} # Style for labels
        self.RecapCC.setLabel("left", "Coeff°", **styles) # Import style for Y label
        self.RecapCC.setLabel("bottom", "Slice", **styles) # Import style for X label
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        
        self.RecapCC.getAxis("left").setTickFont(font) # Apply size of the ticks label
        self.RecapCC.getAxis("left").setStyle(tickTextOffset = 20) # Apply a slight offset
        self.RecapCC.getAxis("bottom").setTickFont(font) # Apply size of the ticks label
        self.RecapCC.getAxis("bottom").setStyle(tickTextOffset = 20) # Apply a slight offset
        
        self.RecapCC.getAxis('left').setTextPen('k') # Set the axis in black
        self.RecapCC.getAxis('bottom').setTextPen('k') # Set the axis in black
        
        self.RecapCC.setBackground(self.parent.color2)
        self.RecapCC.showGrid(x=True, y=True)

    def Pre_notreatment(self): # Steps prior registration is no pretreatment
        self.Pre_treatment_stack_output = np.copy(self.Pre_treatment_stack)
        self.Pre_treatment_stack_output2 = self.Pre_treatment_stack_output
        self.display_Pre_treatment(self.Pre_treatment_stack_output2)
        self.Pre_treatment_stack_output2 = self.Pre_treatment_stack_output2
    
    def Pre_treatment_slice(self): # Apply pretreatment on 1/10 slices at the opening of the sub-gui
        self.Pre_treatment_stack_output = np.copy(self.Pre_treatment_stack)
        self.Pre_treatment_stack_Remout = np.copy(self.Pre_treatment_stack)
        
        if len(self.Pre_treatment_stack_Remout) < 5:
            self.parent.popup_message("Registration","A minimum of 5 images is needed for registration. Please, considerer more images",'icons/Main_icon.png')
            return
        
        else: # Here, remouve outliers, Gaussian blur and sobel factor are applied
            for i in range(0,int(len(self.Pre_treatment_stack)),int(len(self.Pre_treatment_stack)/5)): # Applique pour chaque slice les paramètres du remove outlier
                _, self.Pre_treatment_stack_Remout[i, :, :] =  gf.remove_outliers(self.Pre_treatment_stack[i,:,:], self.radius_Val, self.threshold_Val)
            
            if self.blur_value != 0 :
                for i in range(0,int(len(self.Pre_treatment_stack)),int(len(self.Pre_treatment_stack)/5)):
                    self.Pre_treatment_stack_output[i,:,:] =cv2.GaussianBlur(self.Pre_treatment_stack_Remout[i,:,:],(self.blur_value,self.blur_value),0)
            else :
                self.Pre_treatment_stack_output = self.Pre_treatment_stack_Remout
            
            if self.sobel_value != 0 :
                for i in range(0,int(len(self.Pre_treatment_stack)),int(len(self.Pre_treatment_stack)/5)):
                    self.grad_x = cv2.Sobel(self.Pre_treatment_stack_output[i,:,:],cv2.CV_32F,1,0,ksize=self.sobel_value)
                    self.grad_y = cv2.Sobel(self.Pre_treatment_stack_output[i,:,:],cv2.CV_32F,0,1,ksize=self.sobel_value)
                
                    self.Pre_treatment_stack_output2[i,:,:] = cv2.addWeighted(np.absolute(self.grad_x), 0.5, np.absolute(self.grad_y), 0.5, 0)
            else :   
                self.Pre_treatment_stack_output2 = self.Pre_treatment_stack_output
         
            self.Pre_treatment_stack_slice = self.Pre_treatment_stack_output2[0:-1:int(len(self.Pre_treatment_stack)/5),:,:]
            self.display_Pre_treatment(self.Pre_treatment_stack_slice)
        
    def Pre_treatment(self): # Apply pretreatment on the entire stack
        self.textEdit.ensureCursorVisible()
        self.textEdit.insertPlainText("\n Apply treatment to the entire stack....")
        QApplication.processEvents()
    
        self.progressBar.setValue(0) # Set the initial value of the Progress bar at 0
        self.progressBar.setRange(0, len(self.Pre_treatment_stack)-1) 
    
        self.Pre_treatment_stack_output2 = np.copy(self.Pre_treatment_stack)
    
        for i in range(0,len(self.Pre_treatment_stack[:, 0, 0])): # Applique pour chaque slice les paramètres du remove outlier
        
            QApplication.processEvents()    
            self.ValSlice = i
            self.progression_bar()
        
            _, self.Pre_treatment_stack_output2[i, :, :] =  gf.remove_outliers(self.Pre_treatment_stack[i,:,:], self.radius_Val, self.threshold_Val)
            if self.blur_value != 0 :
                self.Pre_treatment_stack_output2[i,:,:] =cv2.GaussianBlur(self.Pre_treatment_stack_output2[i,:,:],(self.blur_value,self.blur_value),0)
            if self.sobel_value != 0 :
                self.grad_x = cv2.Sobel(self.Pre_treatment_stack_output2[i,:,:],cv2.CV_32F,1,0,ksize=self.sobel_value)
                self.grad_y = cv2.Sobel(self.Pre_treatment_stack_output2[i,:,:],cv2.CV_32F,0,1,ksize=self.sobel_value)
            
                self.Pre_treatment_stack_output2[i,:,:] = cv2.addWeighted(np.absolute(self.grad_x), 0.5, np.absolute(self.grad_y), 0.5, 0)
     
        self.display_Pre_treatment(self.Pre_treatment_stack_output2)
        self.textEdit.insertPlainText("\n The stack is ready for registration.")

    def Pre_treatment_doppel(self): # Apply pretreatment on the doppelganger stack (for co-registration)
        self.doppelganger_stack = np.flip(self.doppelganger_stack, 1)
        self.doppelganger_stack = np.rot90(self.doppelganger_stack, k=1, axes=(2, 1))
        
        self.type = self.doppelganger_stack.dtype

        if self.type == "uint16" or self.type == "uint32":
            self.doppelganger_stack = gf.convertToUint8(self.doppelganger_stack)
               
        self.Pre_treatment_doppelstack = np.copy(self.doppelganger_stack)
        self.Pre_treatment_doppelstack_Remout = np.copy(self.doppelganger_stack)
        
        for i in range(0,len(self.Pre_treatment_doppelstack[:, 0, 0])): # Applique pour chaque slice les paramètres du remove outlier
            _, self.Pre_treatment_doppelstack_Remout[i, :, :] =  gf.remove_outliers(self.Pre_treatment_doppelstack[i,:,:], self.radius_Val, self.threshold_Val)
        
        if self.blur_value != 0 :
            for i in range(0,len(self.Pre_treatment_doppelstack)):
                self.Pre_treatment_doppelstack_output[i,:,:] =cv2.GaussianBlur(self.Pre_treatment_doppelstack_Remout[i,:,:],(self.blur_value,self.blur_value),0)
        else :
            self.Pre_treatment_doppelstack_output = self.Pre_treatment_doppelstack_Remout
        
        if self.sobel_value != 0 :
            for i in range(0,len(self.Pre_treatment_doppelstack)):
                self.grad_x = cv2.Sobel(self.Pre_treatment_doppelstack_output[i,:,:],cv2.CV_32F,1,0,ksize=self.sobel_value)
                self.grad_y = cv2.Sobel(self.Pre_treatment_doppelstack_output[i,:,:],cv2.CV_32F,0,1,ksize=self.sobel_value)
            
                self.Pre_treatment_doppelstack_output2[i,:,:] = cv2.addWeighted(np.absolute(self.grad_x), 0.5, np.absolute(self.grad_y), 0.5, 0)
     
        else :   
            self.Pre_treatment_doppelstack_output2 = self.Pre_treatment_doppelstack

    def Coefficient_estim(self): # Estim corrcoeff between the two first image (using pretreated slices and homography)
        self.Pre_treatment_stack_output = np.copy(self.Pre_treatment_stack)
        self.Pre_treatment_stack_Remout = np.copy(self.Pre_treatment_stack)
        
        for i in range(0,2): # Applique pour chaque slice les paramètres du remove outlier
            _, self.Pre_treatment_stack_Remout[i, :, :] =  gf.remove_outliers(self.Pre_treatment_stack[i,:,:], self.radius_Val, self.threshold_Val)
        
        if self.blur_value != 0 :
            for i in range(0,2):
                self.Pre_treatment_stack_output[i,:,:] =cv2.GaussianBlur(self.Pre_treatment_stack_Remout[i,:,:],(self.blur_value,self.blur_value),0)
        else :
            self.Pre_treatment_stack_output = self.Pre_treatment_stack_Remout
        
        if self.sobel_value != 0 :
            for i in range(0,2):
                self.grad_x = cv2.Sobel(self.Pre_treatment_stack_output[i,:,:],cv2.CV_32F,1,0,ksize=self.sobel_value)
                self.grad_y = cv2.Sobel(self.Pre_treatment_stack_output[i,:,:],cv2.CV_32F,0,1,ksize=self.sobel_value)
            
                self.Pre_treatment_stack_output2[i,:,:] = cv2.addWeighted(np.absolute(self.grad_x), 0.5, np.absolute(self.grad_y), 0.5, 0)
        else :   
            self.Pre_treatment_stack_output2 = self.Pre_treatment_stack_output
             
        self.warp_mode = self.wrapmode
        if self.warp_mode  == cv2.MOTION_HOMOGRAPHY:
            self.warp =  np.eye(3, 3, dtype=np.float32) # Matrice pour le stockage des transformations homographiques
        else :
            self.warp =  np.eye(2, 3, dtype=np.float32) # Matrice pour le stockage des transformations affines

        self.im1 = self.Pre_treatment_stack_output2[self.im_reference,:,:] # Définition de la slice de référence
        self.im2 = self.Pre_treatment_stack_output2[self.im_reference +1,:,:]
        self.criteria = (cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, self.iter_value, self.thres_value) # Stockage des critères pour l'alignement
        
        self.Recap_cc = []

        try :   
            self.cc, self.warp = cv2.findTransformECC(self.im1, self.im2, self.warp, self.warp_mode, self.criteria)
            self.cc = '%.3f'%(self.cc)
        except :
            QApplication.processEvents()
            self.textEdit.insertPlainText("\n Impossible estimation. Try other parameters.")

        self.CC_info.setText('Correlation coefficient: ' + self.cc)

    def Choice_method(self): # Define the program that must be use to perform registration (between sequential, incremental and coregistration)
        self.choice = self.choiceBox.currentText()
        
        if self.choice == 'Sequential':
            self.Registration_seq_step() 
        elif self.choice == 'Incremental':
            self.Registration_incre_step()
        elif self.choice == 'Coregistration':
            self.Registration_coreg_step()

#%% Functions : registration
    def Registration_seq_step(self): # Step to perform sequential registration
        self.choice_transfo = self.transfo_comboBox.currentText()
        self.flagTransformation = self.choice_transfo
        if self.choice_transfo == "Translation" or self.choice_transfo == "Scaled rotation" or self.choice_transfo == "Rigid body" :
            self.Aligned_stack = self.Seq_registration()
            self.AlignedStack.ui.roiBtn.hide()
            self.AlignedStack.ui.histogram.hide()
            self.display_Aligne_treatment(self.Aligned_stack)
            self.textEdit.insertPlainText("\n Sequential registration is complete.")
            self.Push_valid.setEnabled(True) # Validation button is enables 
        else:
            self.Aligned_stack, self.Recap_cc, self.Recap_warp = self.Seq_registration()
            self.AlignedStack.ui.roiBtn.hide()
            self.AlignedStack.ui.histogram.hide()
            self.display_Aligne_treatment(self.Aligned_stack)
            self.drawRecapCC(self.Recap_cc)
            self.textEdit.insertPlainText("\n Sequential registration is complete.")
            self.Push_valid.setEnabled(True) # Validation button is enables 
        
    def Registration_incre_step(self):  # Step to perform incremental registration
        self.choice_transfo = self.transfo_comboBox.currentText()
        self.flagTransformation = self.choice_transfo
        if self.choice_transfo == "Translation" or self.choice_transfo == "Scaled rotation" or self.choice_transfo == "Rigid body" :
            self.Aligned_stack = self.Incre_registration()
            self.AlignedStack.ui.roiBtn.hide()
            self.AlignedStack.ui.histogram.hide()
            self.display_Aligne_treatment(self.Aligned_stack)
            self.textEdit.insertPlainText("\n Incremental registration is complete.")
            self.Push_valid.setEnabled(True) # Validation button is enables 
        else:
            self.Aligned_stack, self.Recap_cc, self.Recap_warp = self.Incre_registration()
            self.AlignedStack.ui.roiBtn.hide()
            self.AlignedStack.ui.histogram.hide()
            self.display_Aligne_treatment(self.Aligned_stack)
            self.drawRecapCC(self.Recap_cc)
            self.textEdit.insertPlainText("\n Incremental registration is complete.")
            self.Push_valid.setEnabled(True) # Validation button is enables 

    def Registration_coreg_step(self):  # Step to perform Co-registration
        self.parent.popup_message("Registration","Only homographic transformation will be used for the Co-registration approach",'icons/Main_icon.png')

        StackLoc_doppel, StackDir_doppel  = gf.getFilePathDialog("Choose the already aligned stack that will be used.' (*.tiff)")
        self.doppelganger_stack = tf.TiffFile(StackLoc_doppel[0]).asarray()       
        self.Pre_treatment_doppel()
        
        self.Aligned_stack, self.Recap_cc, self.Recap_warp = self.coreg_registration()
        self.AlignedStack.ui.roiBtn.hide()
        self.AlignedStack.ui.histogram.hide()
        self.display_Aligne_treatment(self.Aligned_stack)
        self.drawRecapCC(self.Recap_cc)
        self.textEdit.insertPlainText("\n Coregistration is complete.")
        self.Push_valid.setEnabled(True) # Validation button is enables 

    def Seq_registration(self): # Sequential registration
        QApplication.processEvents()
        self.textEdit.insertPlainText("\n ------------------------------------")
        self.textEdit.insertPlainText("\n Sequential registration is running.")
        
        self.Aligned_stack = np.copy(self.expStack)
        
        QApplication.processEvents()
        self.textEdit.insertPlainText("\n Transformation: " + str(self.flagTransformation))
        
        # Cases of Translation - Scaled rotation - Rigid body
        if self.choice_transfo == "Translation":
            sr = StackReg(StackReg.TRANSLATION)
            tmat = sr.register_stack(self.Pre_treatment_stack_output2, axis=0, reference='first',  progress_callback=self.show_progress)
            self.Aligned_stack = sr.transform_stack(self.Aligned_stack)
            return self.Aligned_stack
        elif self.choice_transfo == "Scaled rotation":
            sr = StackReg(StackReg.SCALED_ROTATION)
            tmat = sr.register_stack(self.Pre_treatment_stack_output2, axis=0, reference='first',  progress_callback=self.show_progress)
            self.Aligned_stack = sr.transform_stack(self.Aligned_stack)
            return self.Aligned_stack
        elif self.choice_transfo == "Rigid body":
            sr = StackReg(StackReg.RIGID_BODY)
            tmat = sr.register_stack(self.Pre_treatment_stack_output2, axis=0, reference='first',  progress_callback=self.show_progress)
            self.Aligned_stack = sr.transform_stack(self.Aligned_stack)
            return self.Aligned_stack
        
        # Case of affine and homography
        else:
            self.warp_mode_selection()
            
            self.Aligned_stack = np.zeros((len(self.Pre_treatment_stack_output2), len(self.Pre_treatment_stack_output2[0]), len(self.Pre_treatment_stack_output2[0][0])), dtype = np.float32)

            self.im1 = self.Pre_treatment_stack_output2[self.im_reference,:,:] # Définition de la slice de référence
    
            self.criteria = (cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, self.iter_value, self.thres_value) # Stockage des critères pour l'alignement
            
            self.Recap_cc = []
            
            self.warp_mode = self.wrapmode
            if self.warp_mode  == cv2.MOTION_HOMOGRAPHY:
                self.warp =  np.eye(3, 3, dtype=np.float32) # Matrice pour le stockage des transformations homographiques
                self.Recap_warp = np.zeros((len(self.expStack),len(self.warp),len(self.warp[0])))
            else :
                self.warp =  np.eye(2, 3, dtype=np.float32) # Matrice pour le stockage des transformations affines
                self.Recap_warp = np.zeros((len(self.expStack),len(self.warp),len(self.warp[0])))
    
            try :
                for i in np.arange(0,self.img_number, 1):
    
                    if i == self.im_reference:
                        self.Aligned_stack[i,:,:] = self.expStack[self.im_reference,:,:]
                    else :
                        self.im2 = self.Pre_treatment_stack_output2[i,:,:]
                        # Recherche des transformations à appliquer pour passer de im1 (ref) et im2 (slice)
                        self.cc, self.warp = cv2.findTransformECC(self.im1, self.im2, self.warp, self.warp_mode, self.criteria)
                        
                        # Application des transformations déterminées pour obtenir une slice alignée par rapport à im1
                        self.Recap_warp[i,:,:] = self.warp
                        self.Recap_cc.append(self.cc) 
                        self.cc = '%.3f'%(self.cc)
                        
                        QApplication.processEvents()                    
                        self.ValSlice = i
                        self.progression_bar()
                        
                        self.Recap_cc_IRT = np.vstack(self.Recap_cc)
                        self.drawRecapCC(self.Recap_cc_IRT)
                            
                        if self.warp_mode  == cv2.MOTION_HOMOGRAPHY:
                            self.Aligned_stack[i,:,:] = cv2.warpPerspective (self.expStack[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)          
                            self.Pre_treatment_stack_output2[i,:,:] = cv2.warpPerspective (self.Pre_treatment_stack_output2[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
                        else :
                            self.Aligned_stack[i,:,:] = cv2.warpAffine (self.expStack[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
                            self.Pre_treatment_stack_output2[i,:,:] = cv2.warpAffine (self.Pre_treatment_stack_output2[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
    
            except :
                QApplication.processEvents()
                self.textEdit.insertPlainText("\n Sequential registration failed. Try other parameters.")
            
            self.Recap_cc = np.vstack(self.Recap_cc)
    
            return self.Aligned_stack, self.Recap_cc, self.Recap_warp

    def Incre_registration(self): # Incremental registration
        self.textEdit.insertPlainText("\n ------------------------------------")
        self.textEdit.insertPlainText("\n Incremental registration is running.")
        
        QApplication.processEvents()
        self.textEdit.insertPlainText("\n Transformation: " + str(self.flagTransformation))
        
        self.Aligned_stack = np.copy(self.expStack)
        
        self.choice_transfo = self.transfo_comboBox.currentText()
        self.flagTransformation = self.choice_transfo
        
        # Cases of Translation - Scaled rotation - Rigid body
        if self.choice_transfo == "Translation":
            sr = StackReg(StackReg.TRANSLATION)
            tmat = sr.register_stack(self.Pre_treatment_stack_output2, axis=0, reference='previous',  progress_callback=self.show_progress)
            self.Aligned_stack = sr.transform_stack(self.Aligned_stack)
            return self.Aligned_stack
        elif self.choice_transfo == "Scaled rotation":
            sr = StackReg(StackReg.SCALED_ROTATION)
            tmat = sr.register_stack(self.Pre_treatment_stack_output2, axis=0, reference='previous',  progress_callback=self.show_progress)
            self.Aligned_stack = sr.transform_stack(self.Aligned_stack)
            return self.Aligned_stack
        elif self.choice_transfo == "Rigid body":
            sr = StackReg(StackReg.RIGID_BODY)
            tmat = sr.register_stack(self.Pre_treatment_stack_output2, axis=0, reference='previous',  progress_callback=self.show_progress)
            self.Aligned_stack = sr.transform_stack(self.Aligned_stack)
            return self.Aligned_stack
        
        # Case of affine and homography
        else:
            self.warp_mode_selection()
            
            self.Aligned_stack = np.zeros((len(self.Pre_treatment_stack_output2), len(self.Pre_treatment_stack_output2[0]), len(self.Pre_treatment_stack_output2[0][0])), dtype = np.float32)
    
            self.warp_mode = self.wrapmode
            if self.warp_mode  == cv2.MOTION_HOMOGRAPHY:
                self.warp =  np.eye(3, 3, dtype=np.float32) # Matrice pour le stockage des transformations homographiques
                self.Recap_warp = np.zeros((len(self.expStack),len(self.warp),len(self.warp[0])))
            else :
                self.warp =  np.eye(2, 3, dtype=np.float32) # Matrice pour le stockage des transformations affines
                self.Recap_warp = np.zeros((len(self.expStack),len(self.warp),len(self.warp[0])))
    
            self.im1 = self.Pre_treatment_stack_output2[self.im_reference,:,:] # Définition de la slice de référence
    
            self.criteria = (cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, self.iter_value, self.thres_value) # Stockage des critères pour l'alignement
            
            self.Recap_cc = []
    
            try :
                for i in np.arange(0,self.img_number, 1):
    
                    if i == self.im_reference:
                        self.Aligned_stack[i,:,:] = self.expStack[self.im_reference,:,:]
                    else :
                        self.im1bis = self.Pre_treatment_stack_output2[i-1,:,:] # Label de la slice de référence (glissante)
                        self.im2 = self.Pre_treatment_stack_output2[i,:,:]
                        # Recherche des transformations à appliquer pour passer de im1 (ref) et im2 (slice)
                        self.cc, self.warp = cv2.findTransformECC(self.im1bis, self.im2, self.warp, self.warp_mode, self.criteria)
                        
                        # Application des transformations déterminées pour obtenir une slice alignée par rapport à im1
                        self.Recap_warp[i,:,:] = self.warp
                        self.Recap_cc.append(self.cc) 
                        self.cc = '%.3f'%(self.cc)
    
                        QApplication.processEvents()
                        self.ValSlice = i
                        self.progression_bar()
                        
                        self.Recap_cc_IRT = np.vstack(self.Recap_cc)
                        self.drawRecapCC(self.Recap_cc_IRT)
                            
                        if self.warp_mode  == cv2.MOTION_HOMOGRAPHY:
                            self.Aligned_stack[i,:,:] = cv2.warpPerspective (self.expStack[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
                            self.Pre_treatment_stack_output2[i,:,:] = cv2.warpPerspective (self.Pre_treatment_stack_output2[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
                        else :
                            self.Aligned_stack[i,:,:] = cv2.warpAffine (self.expStack[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
                            self.Pre_treatment_stack_output2[i,:,:] = cv2.warpAffine (self.Pre_treatment_stack_output2[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
                    
            except :
                QApplication.processEvents()
                self.textEdit.insertPlainText("\n Incremental registration failed. Try other parameters.")
            
            self.Recap_cc = np.vstack(self.Recap_cc)
    
            return self.Aligned_stack, self.Recap_cc, self.Recap_warp

    def coreg_registration(self): # Co-registration (homographic transformation only)
        self.textEdit.insertPlainText("\n ------------------------------------")
        self.textEdit.insertPlainText("\n Coregistration is running.")
        
        self.Aligned_stack = np.zeros((len(self.Pre_treatment_stack_output2), len(self.Pre_treatment_stack_output2[0]), len(self.Pre_treatment_stack_output2[0][0])), dtype = np.float32)
        
        self.wrapmode  = cv2.MOTION_HOMOGRAPHY
        self.textEdit.insertPlainText("\n CoReg°: homography will be used.")
        self.flagTransformation = "Homography"
        
        if self.warp_mode  == cv2.MOTION_HOMOGRAPHY:
            self.warp =  np.eye(3, 3, dtype=np.float32) # Matrice pour le stockage des transformations homographiques
            self.Recap_warp = np.zeros((len(self.expStack),len(self.warp),len(self.warp[0])))
        else :
            self.warp =  np.eye(2, 3, dtype=np.float32) # Matrice pour le stockage des transformations affines
            self.Recap_warp = np.zeros((len(self.expStack),len(self.warp),len(self.warp[0])))

        self.criteria = (cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, self.iter_value, self.thres_value) # Stockage des critères pour l'alignement
        
        self.Recap_cc = []
        
        try :
            for i in np.arange(0,self.img_number, 1):

                if i == self.im_reference:
                    self.Aligned_stack[i,:,:] = self.expStack[self.im_reference,:,:]
                else :
                    self.im_stack = self.Pre_treatment_stack_output2[i,:,:] # Label de la slice à aligner (glissante)
                    self.im_doppel = self.Pre_treatment_doppelstack_output2[i,:,:] # Label de la slice de référence (glissante)

                    # Recherche des transformations à appliquer pour passer de im1 (ref) et im2 (slice)
                    self.cc, self.warp = cv2.findTransformECC(self.im_doppel, self.im_stack, self.warp, self.warp_mode, self.criteria)
                        
                    # Application des transformations déterminées pour obtenir une slice alignée par rapport à im1
                    self.Recap_warp[i,:,:] = self.warp
                    self.Recap_cc.append(self.cc) 
                    self.cc = '%.3f'%(self.cc)

                    QApplication.processEvents()    
                    self.ValSlice = i
                    self.progression_bar()
                    
                    self.Recap_cc_IRT = np.vstack(self.Recap_cc)
                    self.drawRecapCC(self.Recap_cc_IRT)
                            
                    if self.warp_mode  == cv2.MOTION_HOMOGRAPHY:
                        self.Aligned_stack[i,:,:] = cv2.warpPerspective (self.expStack[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
                        self.Pre_treatment_stack_output2[i,:,:] = cv2.warpPerspective (self.Pre_treatment_stack_output2[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
                    else :
                        self.Aligned_stack[i,:,:] = cv2.warpAffine (self.expStack[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
                        self.Pre_treatment_stack_output2[i,:,:] = cv2.warpAffine (self.Pre_treatment_stack_output2[i,:,:], self.warp, ( self.stack_width,self.stack_height), flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP)
                    
        except :
            self.textEdit.insertPlainText("\n Coregistration failed. Try other parameters.")
        
        self.Recap_cc = np.vstack(self.Recap_cc)

        return self.Aligned_stack, self.Recap_cc, self.Recap_warp

#%% Functions : final treatment
    def Crop_data(self): # Step to crop stack
        self.Cropped_min_stack(self.Aligned_stack)
        
        self.Cropped_stack = self.Aligned_stack[:,self.Selection[1]:self.Selection[3],self.Selection[0]:self.Selection[2]]   # Application sur la stack alignée pour rogner
        self.Cropped_pre_treatment_stack = self.Pre_treatment_stack_output2[:,self.Selection[1]:self.Selection[3],self.Selection[0]:self.Selection[2]]   # Application sur la stack alignée pour rogner

        self.textEdit.insertPlainText("\n Data has been cropped.")
        
        self.AlignedStack.ui.roiBtn.hide()
        self.AlignedStack.ui.histogram.hide()
        self.display_Aligne_treatment(self.Cropped_stack)
        
    def Cropped_min_stack(self, stack): # Step to define the position of the largest rectangle to be cropped
        self.Min_stack = np.min(stack,0) # Define min map
        self.Min_stack = self.Mask_min(self.Min_stack,0) # If value is not 0, else it became 1
        self.Min_stack = self.Min_stack.astype("bool") # Convert to Boolean
        self.Min_stack = sci.binary_fill_holes(self.Min_stack).astype("bool") # Fill holes to optimize the cropping step
    
        self.Selection = lir.lir(self.Min_stack) # Define the dimension of the largest rectangle 
        
    def Mask_min(self,stack,threshold): # If value is not 0, else it became 1
        self.Mask = np.zeros((len(stack),len(stack[0])))
        self.Mask[stack > threshold] = 1

        return self.Mask
        
    def extract_align_stack(self): # Extract registered stack to the main gui
    
        if self.flagCrop == "No" :
            self.Crop_data()
            
            self.parent.Current_stack = np.copy(self.Cropped_stack)
            self.parent.StackList.append(self.Cropped_stack)
            
            Combo_text = '\u2022 Registered stack'
            # Combo_text = '\u2022 Registered stack. Transformation : ' + self.flagTransformation + ' ; ' + self.choice
            Combo_data = self.Cropped_stack
            self.parent.choiceBox.addItem(Combo_text, Combo_data)
    
            self.parent.displayExpStack(self.parent.Current_stack)
            
            self.parent.Info_box.ensureCursorVisible()
            self.parent.Info_box.insertPlainText("\n \u2022 Registered stack has been exported.")
            
            self.close()
            
        elif self.flagCrop == "Yes" :
            self.parent.Current_stack = np.copy(self.Aligned_stack)
            self.parent.StackList.append(self.Aligned_stack)
            
            Combo_text = '\u2022 Registered stack. Transformation : ' + self.flagTransformation + ' ; ' + self.choice
            Combo_data = self.Aligned_stack
            self.parent.choiceBox.addItem(Combo_text, Combo_data)
    
            self.parent.displayExpStack(self.parent.Current_stack)
            
            self.parent.Info_box.ensureCursorVisible()
            self.parent.Info_box.insertPlainText("\n \u2022 Registered stack has been exported.")
            
            self.close()
        
#%% Functions : data visualization
    def displayExpStack(self, series):
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()

        view = self.expSeries.getView()
        state = view.getState()        
        self.expSeries.setImage(series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.expSeries.timeLine.setPen(color=self.parent.color3, width=15)
        self.expSeries.frameTicks.setPen(color=self.parent.color1, width=5)
        self.expSeries.frameTicks.setYRange((0, 1))

        s = self.expSeries.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 
        
    def display_Aligne_treatment(self, Series): # Affichage de la stack alignée
        self.AlignedStack.ui.histogram.hide()
        self.AlignedStack.ui.roiBtn.hide()
        self.AlignedStack.ui.menuBtn.hide()
        
        view = self.AlignedStack.getView()
        state = view.getState()        
        self.AlignedStack.setImage(Series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.AlignedStack.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.AlignedStack.timeLine.setPen(color=self.parent.color3, width=15)
        self.AlignedStack.frameTicks.setPen(color=self.parent.color1, width=5)
        self.AlignedStack.frameTicks.setYRange((0, 1))

        s = self.AlignedStack.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 
       
    def display_Pre_treatment(self, Series):  # Affichage de la stack de pré-traitement
    
        self.pretreatment.setImage(Series) 
    
        self.pretreatment.clear()
        self.pretreatment.ui.menuBtn.hide()
        self.pretreatment.ui.roiBtn.hide()
        
        view = self.pretreatment.getView()
        state = view.getState()        
        self.pretreatment.setImage(Series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.pretreatment.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.pretreatment.timeLine.setPen(color=self.parent.color3, width=15)
        self.pretreatment.frameTicks.setPen(color=self.parent.color1, width=5)
        self.pretreatment.frameTicks.setYRange((0, 1))

        s = self.pretreatment.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 
        
        histplot = self.pretreatment.getHistogramWidget()
        histplot.setBackground(self.parent.color1)
        
        histplot.region.setBrush(pg.mkBrush(self.parent.color5 + (120,)))
        histplot.region.setHoverBrush(pg.mkBrush(self.parent.color5 + (60,)))
        histplot.region.pen = pg.mkPen(self.parent.color5)
        histplot.region.lines[0].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.region.lines[1].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.fillHistogram(color = self.parent.color5)        
        histplot.autoHistogramRange()
        
    def progression_bar(self): # Use to display and manage Progress bar
        self.prgbar = self.ValSlice
        self.progressBar.setValue(self.prgbar)
        
    def show_progress(self,current_iteration, end_iteration): # For Translation - Scaled rotation - Rigid body
        QApplication.processEvents()                    
        self.ValSlice = current_iteration
        self.progression_bar()



# -*- coding: utf-8 -*-
"""
Created on Wed Nov 22 23:26:20 2023

@author: clanglois1
"""
import os
import sys

from inspect import getsourcefile
from os.path import abspath

import numpy as np
import pyqtgraph as pg
import tifffile as tf
import time

from PyQt5.QtWidgets import QApplication, QWidget, QLabel, QVBoxLayout, QPushButton, QMessageBox

from PyQt5 import QtCore, QtGui
from PyQt5.QtGui import QPixmap

from inichord import general_functions as gf

from skimage import morphology, filters, exposure
from scipy import ndimage as ndi

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/Restored_v1_TSG.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self, parent):
        super().__init__()
        
        self.setupUi(self)
        self.parent = parent
        
        self.setWindowIcon(QtGui.QIcon('icons/Restored_Icons.png'))
        
        self.flag_restored_only = False # To display Otsu n°1 value or not
        self.flag_restored_struc = False # To display grain labels value or not
        self.flag_overlay = False # No metric at the beginning
        
        self.x = 0
        self.y = 0

        self.crosshair_v1= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h1 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.crosshair_v2= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h2 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.crosshair_v3= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h3 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.crosshair_v4= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h4 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.crosshair_v5= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h5 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.crosshair_v6= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h6 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.proxy1 = pg.SignalProxy(self.KADSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy2 = pg.SignalProxy(self.KADSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)
        
        self.proxy3 = pg.SignalProxy(self.FiltKADSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy4 = pg.SignalProxy(self.FiltKADSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)

        self.proxy5 = pg.SignalProxy(self.Otsu1Series.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy6 = pg.SignalProxy(self.Otsu1Series.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)

        self.proxy7 = pg.SignalProxy(self.Binary1Series.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy8 = pg.SignalProxy(self.Binary1Series.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)

        self.proxy9 = pg.SignalProxy(self.Otsu2Series.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy10 = pg.SignalProxy(self.Otsu2Series.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)

        self.proxy11 = pg.SignalProxy(self.Binary2Series.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy12 = pg.SignalProxy(self.Binary2Series.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)

        self.OpenData.clicked.connect(self.loaddata) # To load a map data
        self.ComputeClass_bttn.clicked.connect(self.Otsu1) # Computation of the Otsu 1 (classes creation)
        self.Threshold_bttn.clicked.connect(self.Binary_1) # Computation of the Otsu thresholding 1
        self.ComputeClass_bttn_2.clicked.connect(self.Otsu2) # Computation of the Otsu 2 (classes creation) 
        self.Threshold_bttn_2.clicked.connect(self.Binary_2) # Computation of the Otsu 2 thresholding
        self.Save_bttn.clicked.connect(self.Save_results) # To change the displayed maps
        self.Full_Run_bttn.clicked.connect(self.FullRun) # Run all parameters as defined
        
        self.PresetBox.currentTextChanged.connect(self.auto_set) # Run the defined autoset 
        self.spinBox_filter.valueChanged.connect(self.Filter_changed) # Apply filter modification 
        
        self.ChoiceBox.currentTextChanged.connect(self.ViewLabeling) # Change the displayed map
        
        self.defaultIV() # Hide the PlotWidget until a data has been loaded
        self.mouseLock.setVisible(False)
        
        # Icons sizes management for QMessageBox
        self.pixmap = QPixmap("icons/Restored_Icons.png")
        self.pixmap = self.pixmap.scaled(100, 100)

        try:
            if hasattr(parent, 'KAD') : # Choice of KAD if only available
                self.InitKAD_map = parent.KAD
                self.flag_KAD = True
                
            if hasattr(parent, 'contour_map') : # Choice of contour map if only avalaible
                self.InitKAD_map = parent.contour_map
                self.flag_KAD = False
                
            if hasattr(parent, 'KAD') and hasattr(parent, 'contour_map'):
                self.show_choice_message() # Choice of the map if the two are available
                
            self.StackDir = self.parent.StackDir
            self.run_init_computation()
        except:
            pass
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        # Control window position and dimensions
        self.move(int(geometry.width() * 0.05), int(geometry.height() * 0.05))
        self.resize(int(geometry.width() * 0.9), int(geometry.height() * 0.8))
        self.screen = screen

#%% Functions
    def show_choice_message(self): # Qmessage box for the try import at the initialization 
        msg_box = QMessageBox()
        msg_box.setWindowTitle("Choice of the data")
        msg_box.setText("Data to use :")
    
        btn_kad = msg_box.addButton("KAD map", QMessageBox.ActionRole)
        btn_contour = msg_box.addButton("Contour map", QMessageBox.ActionRole)

        msg_box.exec_()
    
        if msg_box.clickedButton() == btn_kad:
            self.InitKAD_map = self.parent.KAD
            self.flag_KAD = True
        elif msg_box.clickedButton() == btn_contour:
            self.InitKAD_map = self.parent.contour_map
            self.flag_KAD = False

    def data_choice(self): # Allow to apply other treatment depending if the data is a KAD one
        msg = QMessageBox.question(self, 'Restored grains', 'Is it a KAD data ?')
        if msg == QMessageBox.Yes:
            self.flag_KAD = True
        if msg == QMessageBox.No:
            self.flag_KAD = False

    def loaddata(self):
        self.defaultIV() # Hide the PlotWidget until a data has been loaded
        self.ChoiceBox.clear() # Clear the QComboBox

        self.StackLoc, self.StackDir = gf.getFilePathDialog("Open 2D map (*.tiff)") 
        
        checkimage = tf.TiffFile(self.StackLoc[0]).asarray() # Check for dimension. If 2 dimensions : 2D array. If 3 dimensions : stack of images
        if checkimage.ndim != 2: # Check if the data is a 2D array
            self.parent.popup_message("Restored grain determination","Please import a 2D array",'icons/Restored_Icons.png')
            return
        
        self.data_choice()
        
        self.InitKAD_map = tf.TiffFile(self.StackLoc[0]).asarray()
        self.InitKAD_map = np.flip(self.InitKAD_map, 0)
        self.InitKAD_map = np.rot90(self.InitKAD_map, k=1, axes=(1, 0))
        
        self.InitKAD_map = np.nan_to_num(self.InitKAD_map) # Exclude NaN value if needed
        self.InitKAD_map = (self.InitKAD_map - np.min(self.InitKAD_map)) / (np.max(self.InitKAD_map) - np.min(self.InitKAD_map)) # Normalization step
        self.InitKAD_map = exposure.equalize_adapthist(self.InitKAD_map, kernel_size=None, clip_limit=0.01, nbins=256) # CLAHE step
        
        self.displayExpKAD(self.InitKAD_map) # Display of the map
        
        self.auto_set() # Allow different preset of parameters to be used.
        
    def run_init_computation(self):
        self.InitKAD_map = np.nan_to_num(self.InitKAD_map) # Exclude NaN value if needed
        self.InitKAD_map = (self.InitKAD_map - np.min(self.InitKAD_map)) / (np.max(self.InitKAD_map) - np.min(self.InitKAD_map)) # Normalization step
        self.InitKAD_map = exposure.equalize_adapthist(self.InitKAD_map, kernel_size=None, clip_limit=0.01, nbins=256) # CLAHE step
        
        self.displayExpKAD(self.InitKAD_map) # Display of the map
        
        self.auto_set() # Default set is "Default"

    def auto_set(self):
        self.Preset_choice = self.PresetBox.currentText()
        
        if self.Preset_choice == "Default":
            self.spinBox_filter.setValue(1) # Filter value
            self.ClassBox.setValue(4) # Number of classes for first Otsu
            self.ThresholdBox.setValue(1) # Threshold value for first Otsu
            self.ClassBox2.setValue(4) # Number of classes for second Otsu
            self.ThresholdBox_2.setValue(1) # Threshold value for last Otsu

            self.Filter_changed() # Compute the filtered map
            self.Otsu1() # Compute a first Otsu map
            self.Binary_1() # Compute a first binary map
            self.Otsu2() # Compute the second Otsu
            self.Binary_2() # Compute the final thresholding
        
        elif self.Preset_choice == "Undeformed sample":
            self.spinBox_filter.setValue(1) # Filter value
            self.ClassBox.setValue(5) # Number of classes for first Otsu
            self.ThresholdBox.setValue(1) # Threshold value for first Otsu
            self.ClassBox2.setValue(5) # Number of classes for second Otsu
            self.ThresholdBox_2.setValue(1) # Threshold value for last Otsu

            self.Filter_changed() # Compute the filtered map
            self.Otsu1() # Compute a first Otsu map
            self.Binary_1() # Compute a first binary map
            self.Otsu2() # Compute the second Otsu
            self.Binary_2() # Compute the final thresholding
            
        elif self.Preset_choice == "Slightly deformed sample":
            self.spinBox_filter.setValue(2) # Filter value
            self.ClassBox.setValue(5) # Number of classes for first Otsu
            self.ThresholdBox.setValue(1) # Threshold value for first Otsu
            self.ClassBox2.setValue(6) # Number of classes for second Otsu
            self.ThresholdBox_2.setValue(1) # Threshold value for last Otsu
            
            self.Filter_changed() # Compute the filtered map
            self.Otsu1() # Compute a first Otsu map
            self.Binary_1() # Compute a first binary map
            self.Otsu2() # Compute the second Otsu
            self.Binary_2() # Compute the final thresholding
            
        elif self.Preset_choice == "Degraded and slightly deformed sample":
            self.spinBox_filter.setValue(2) # Filter value
            self.ClassBox.setValue(5) # Number of classes for first Otsu
            self.ThresholdBox.setValue(3) # Threshold value for first Otsu
            self.ClassBox2.setValue(5) # Number of classes for second Otsu
            self.ThresholdBox_2.setValue(3) # Threshold value for last Otsu
            
            self.Filter_changed() # Compute the filtered map
            self.Otsu1() # Compute a first Otsu map
            self.Binary_1() # Compute a first binary map
            self.Otsu2() # Compute the second Otsu
            self.Binary_2() # Compute the final thresholding
            
        elif self.Preset_choice == "Deformed sample":
            self.spinBox_filter.setValue(1) # Filter value
            self.ClassBox.setValue(6) # Number of classes for first Otsu
            self.ThresholdBox.setValue(3) # Threshold value for first Otsu
            self.ClassBox2.setValue(6) # Number of classes for second Otsu
            self.ThresholdBox_2.setValue(3) # Threshold value for last Otsu
            
            self.Filter_changed() # Compute the filtered map
            self.Otsu1() # Compute a first Otsu map
            self.Binary_1() # Compute a first binary map
            self.Otsu2() # Compute the second Otsu
            self.Binary_2() # Compute the final thresholding

    def Filter_changed(self): # Allow the modification of the map with different filtering and values
        self.Filter_choice = self.FilterBox.currentText()
        
        if self.Filter_choice == "Mean filter":
            self.spinBox_filter.setRange(0,10)
            self.spinBox_filter.setSingleStep(1)
            
            self.FilteredKAD_map = np.copy(self.InitKAD_map)
            self.Filter_value = int(self.spinBox_filter.value())
            self.FilteredKAD_map = filters.gaussian(self.FilteredKAD_map, self.Filter_value)
        
        elif self.Filter_choice == "Butterworth (LP) filter":
            self.spinBox_filter.setRange(0.1,0.5)
            self.spinBox_filter.setSingleStep(0.1)
            
            self.FilteredKAD_map = np.copy(self.InitKAD_map)
            self.Filter_value = self.spinBox_filter.value()
            self.FilteredKAD_map = filters.butterworth(self.FilteredKAD_map,self.Filter_value,False,8)
            
        self.displayFilteredKAD(self.FilteredKAD_map) # Display the map after load

    def Otsu1(self): # Segment map into classes
        self.Otsu1_Value = self.ClassBox.value()

        # Segmentation of the intensities for a given number of classes
        thresholds = filters.threshold_multiotsu(self.FilteredKAD_map, classes = self.Otsu1_Value) # Definition of the threshold values
        self.regions = np.digitize(self.FilteredKAD_map, bins=thresholds) # Using the threshold values, we generate the regions.

        self.displayOtsu1(self.regions) # Display the Otsu map

    def Binary_1(self): # Binarization of the Otsu map for a given threshold level
        self.Binary1_Value = self.ThresholdBox.value()
        
        self.regions2 = np.copy(self.regions)

        var_up = np.where(self.regions >= self.Binary1_Value) # Search for every value higher or equal to threshold
        var_down = np.where(self.regions < self.Binary1_Value) # Search for every value below the threshold
        self.regions2[var_up] = 1 # Replace values by 1 ==> Binary image created
        self.regions2[var_down] = 0 # Replace values by 1 ==> Binary image created

        self.regions3 = ndi.binary_closing(self.regions2) # Closing step 
        self.binary_regions = 1-(ndi.binary_dilation(self.regions3, iterations = 1)) # Dilation to increase connectivity

        self.binary_regions = ndi.binary_opening(self.binary_regions, iterations = 2) # Opening for better result

        self.displayBinary1(self.binary_regions) # Display the thresholded map

    def Otsu2(self): # Second Otsu (on the computed distance map)
        self.Otsu2_Value = self.ClassBox2.value()
        self.binary_regions_distance = ndi.distance_transform_edt(1-self.binary_regions)
        
        # Segmentation of the intensities for a given number of classes
        thresholds = filters.threshold_multiotsu(self.binary_regions_distance, classes = self.Otsu2_Value) # Definition of the threshold values
        self.regions_distance = np.digitize(self.binary_regions_distance, bins=thresholds) # Using the threshold values, we generate the regions.

        self.displayOtsu2(self.regions_distance) # Display the Otsu map

    def Binary_2(self):
        self.Binary2_Value = self.ThresholdBox_2.value()
        
        self.binary_regions2 = np.copy(self.regions_distance)
        var = np.where(self.binary_regions2 > self.Binary2_Value) # Every pixel with value > Binary2_Value is considerered
        self.binary_regions2[var] = 5 # Replace thoses pixel by a contant value of 5
      
        self.restored_grains = np.zeros((len(self.binary_regions2),len(self.binary_regions2[0])))
        self.restored_grains[var] = 5
        self.restored_grains = ndi.binary_fill_holes(self.restored_grains).astype("bool") # Fill holes processing # Restored grain 

        var = np.where(self.restored_grains == True) # Every pixel with value > Binary2_Value is considerered
        self.binary_regions2[var] = 5
        
        # Creation of the overlay map: map and restored pixels location
        var = np.where(self.restored_grains == True)
        self.overlay_KAD_restored = np.copy(self.InitKAD_map)
        self.overlay_KAD_restored[var] = 1   
        
        # Creation of items in the QComboBox
        self.ChoiceBox.clear() 
        
        Combo_text = 'Restored grains and structure'
        Combo_data = self.binary_regions2
        self.ChoiceBox.addItem(Combo_text, Combo_data)
        
        Combo_text = 'Restored grains only'
        Combo_data = self.restored_grains
        self.ChoiceBox.addItem(Combo_text, Combo_data)

        Combo_text = 'Overlay map-restored grains'
        Combo_data = self.overlay_KAD_restored
        self.ChoiceBox.addItem(Combo_text, Combo_data)
        
        # Run the restored fraction computation
        self.Compute_restoredFrac()
        
    def Compute_restoredFrac(self):       
        Dilate = morphology.binary_dilation(self.restored_grains) # Apply a binary dilatation of the pixels
        Erode = morphology.binary_erosion(self.restored_grains) # Apply a binary dilatation of the pixels

        fraction_base = (np.sum(self.restored_grains == True)/self.restored_grains.size)*100 # Computation the restored fraction
        fraction_lowerbound = (np.sum(Erode == True)/Erode.size)*100
        fraction_upperbound = (np.sum(Dilate == True)/Dilate.size)*100

        var = [fraction_base,fraction_lowerbound,fraction_upperbound]
        self.fraction_mean = np.round(np.mean(var),2)
        self.fraction_std = np.round(np.std(var),2)
        
        self.Restored_infos.setText('Restored fraction: ' + str(self.fraction_mean) + ' \u00B1 ' + str(self.fraction_std) + ' % ')

    def ViewLabeling(self):
        self.view_choice = self.ChoiceBox.currentText()
        
        self.flag_restored_only = False # To display Otsu n°1 value or not
        self.flag_restored_struc = False # To display grain labels value or not
        self.flag_overlay = False # No metric at the beginning

        if self.view_choice == "Restored grains only":
            self.displayBinary2(self.restored_grains) # Display the KAD map after load
            self.flag_restored_only = True
            
        if self.view_choice == "Restored grains and structure":
            self.displayBinary2(self.binary_regions2) # Display the KAD map after load
            self.flag_restored_struc = True
            
        if self.view_choice == "Overlay map-restored grains":
            self.displayBinary2(self.overlay_KAD_restored) # Display the KAD map after load
            self.flag_overlay = True
            
    def FullRun(self):
        # We take all the actual parameters and we compute the data
        self.Otsu1() # Compute a first Otsu map
        self.Binary_1() # Compute a first binary map
        self.Otsu2() # Compute the second Otsu
        self.Binary_2() # Compute the final thresholding
        
    def Save_results(self):
        ti = time.strftime("%Y-%m-%d__%Hh-%Mm-%Ss")
        
        directory = "Restored_grain_determination_" + ti
        PathDir = os.path.join(self.StackDir, directory) 
        os.mkdir(PathDir)  
        
        tf.imwrite(PathDir + '/Map_CLAHE.tiff', np.rot90(np.flip(self.InitKAD_map, 0), k=1, axes=(1, 0)))
        tf.imwrite(PathDir + '/Filtered_map.tiff', np.rot90(np.flip(self.FilteredKAD_map, 0), k=1, axes=(1, 0)))  
        tf.imwrite(PathDir + '/Otsu.tiff', np.rot90(np.flip(self.regions, 0), k=1, axes=(1, 0)).astype('float32')) 
        tf.imwrite(PathDir + '/Binary_Otsu.tiff', np.rot90(np.flip(self.binary_regions, 0), k=1, axes=(1, 0)))
        tf.imwrite(PathDir + '/Otsu_2.tiff', np.rot90(np.flip(self.regions_distance, 0), k=1, axes=(1, 0)).astype('float32')) 
        tf.imwrite(PathDir + '/Threshold_2.tiff', np.rot90(np.flip(self.binary_regions2, 0), k=1, axes=(1, 0)).astype('float32'))  # Map with restored pixel = 5
        tf.imwrite(PathDir + '/Restored_grains_only.tiff', np.rot90(np.flip(self.restored_grains, 0), k=1, axes=(1, 0))) # Map with restored pixel = 5
        tf.imwrite(PathDir + '/Overlay_map_restored.tiff', np.rot90(np.flip(self.overlay_KAD_restored, 0), k=1, axes=(1, 0))) # Map with restored pixel = 5
        
        with open(PathDir + '\Restored grains determination.txt', 'w') as file:
            file.write("KAD data: " + str(self.flag_KAD))
            file.write("\nFiltering parameter: " + str(self.Filter_choice) + " - " + (str(self.Filter_value)))   
            file.write("\nOtsu n°1 class: "+ str(self.Otsu1_Value) + "\nThresholded n°1 classes (keep values equal or higher than): " + str(self.Binary1_Value))   
            file.write("\nOtsu n°2 class: "+ str(self.Otsu2_Value) + "\nThresholded n°2 classes (keep values higher than): " + str(self.Binary2_Value))     
            file.write("\nRestored fraction: "+ str(self.fraction_mean) + ' \u00B1 ' + str(self.fraction_std) + ' % ')

        self.parent.popup_message("Restored grain determination","Saving process is over.",'icons/Restored_Icons.png')

    def displayExpKAD(self, series): # Display of initial KAD map
        self.KADSeries.addItem(self.crosshair_v1, ignoreBounds=True)
        self.KADSeries.addItem(self.crosshair_h1, ignoreBounds=True) 
        
        self.KADSeries.ui.histogram.hide()
        self.KADSeries.ui.roiBtn.hide()
        self.KADSeries.ui.menuBtn.hide()
        
        view = self.KADSeries.getView()
        state = view.getState()        
        self.KADSeries.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        self.KADSeries.autoRange()
        
    def displayFilteredKAD(self, series): # Display the filtered KAD map
        self.FiltKADSeries.addItem(self.crosshair_v2, ignoreBounds=True)
        self.FiltKADSeries.addItem(self.crosshair_h2, ignoreBounds=True) 
        
        self.FiltKADSeries.ui.histogram.hide()
        self.FiltKADSeries.ui.roiBtn.hide()
        self.FiltKADSeries.ui.menuBtn.hide()
        
        view = self.FiltKADSeries.getView()
        state = view.getState()        
        self.FiltKADSeries.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        # self.FiltKADSeries.autoRange()
        
    def displayOtsu1(self, series): # Display Otsu 1 regions
        self.Otsu1Series.addItem(self.crosshair_v3, ignoreBounds=True)
        self.Otsu1Series.addItem(self.crosshair_h3, ignoreBounds=True) 
        
        self.Otsu1Series.ui.histogram.show()
        self.Otsu1Series.ui.roiBtn.hide()
        self.Otsu1Series.ui.menuBtn.hide()
        
        view = self.Otsu1Series.getView()
        state = view.getState()        
        self.Otsu1Series.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        histplot = self.Otsu1Series.getHistogramWidget()
        histplot.setBackground(self.parent.color1)
        
        histplot.region.setBrush(pg.mkBrush(self.parent.color5 + (120,)))
        histplot.region.setHoverBrush(pg.mkBrush(self.parent.color5 + (60,)))
        histplot.region.pen = pg.mkPen(self.parent.color5)
        histplot.region.lines[0].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.region.lines[1].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.fillHistogram(color = self.parent.color5)        
        histplot.autoHistogramRange()
        
        self.Otsu1Series.setColorMap(pg.colormap.get('viridis'))
        
    def displayOtsu2(self, series): # Display the distance Otsu regions after Binary1
        self.Otsu2Series.addItem(self.crosshair_v4, ignoreBounds=True)
        self.Otsu2Series.addItem(self.crosshair_h4, ignoreBounds=True) 
        
        self.Otsu2Series.ui.histogram.show()
        self.Otsu2Series.ui.roiBtn.hide()
        self.Otsu2Series.ui.menuBtn.hide()
        
        view = self.Otsu2Series.getView()
        state = view.getState()        
        self.Otsu2Series.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        histplot = self.Otsu2Series.getHistogramWidget()
        histplot.setBackground(self.parent.color1)
        
        histplot.region.setBrush(pg.mkBrush(self.parent.color5 + (120,)))
        histplot.region.setHoverBrush(pg.mkBrush(self.parent.color5 + (60,)))
        histplot.region.pen = pg.mkPen(self.parent.color5)
        histplot.region.lines[0].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.region.lines[1].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.fillHistogram(color = self.parent.color5)        
        histplot.autoHistogramRange()
        
        self.Otsu2Series.setColorMap(pg.colormap.get('viridis'))
                
    def displayBinary1(self, series): # Display first thresholding response
        self.Binary1Series.addItem(self.crosshair_v5, ignoreBounds=True)
        self.Binary1Series.addItem(self.crosshair_h5, ignoreBounds=True) 
        
        self.Binary1Series.ui.histogram.hide()
        self.Binary1Series.ui.roiBtn.hide()
        self.Binary1Series.ui.menuBtn.hide()
        
        view = self.Binary1Series.getView()
        state = view.getState()        
        self.Binary1Series.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
    def displayBinary2(self, series): # Display final results
        self.Binary2Series.addItem(self.crosshair_v6, ignoreBounds=True)
        self.Binary2Series.addItem(self.crosshair_h6, ignoreBounds=True) 
        
        self.Binary2Series.ui.histogram.show()
        self.Binary2Series.ui.roiBtn.hide()
        self.Binary2Series.ui.menuBtn.hide()
        
        view = self.Binary2Series.getView()
        state = view.getState()        
        self.Binary2Series.setImage(series) 
        view.setState(state)
        view.setBackgroundColor(self.parent.color1)
        
        histplot = self.Binary2Series.getHistogramWidget()
        histplot.setBackground(self.parent.color1)
        
        histplot.region.setBrush(pg.mkBrush(self.parent.color5 + (120,)))
        histplot.region.setHoverBrush(pg.mkBrush(self.parent.color5 + (60,)))
        histplot.region.pen = pg.mkPen(self.parent.color5)
        histplot.region.lines[0].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.region.lines[1].setPen(pg.mkPen(self.parent.color5, width=2))
        histplot.fillHistogram(color = self.parent.color5)        
        histplot.autoHistogramRange()
        
        self.Binary2Series.setColorMap(pg.colormap.get('CET-D7'))

    def defaultIV(self):
        # KADSeries: Initial KAD
        self.KADSeries.ui.histogram.hide()
        self.KADSeries.ui.roiBtn.hide()
        self.KADSeries.ui.menuBtn.hide()
        
        view = self.KADSeries.getView()
        view.setBackgroundColor(self.parent.color1)
        
        # FiltKADSeries: KAD after filtering
        self.FiltKADSeries.ui.histogram.hide()
        self.FiltKADSeries.ui.roiBtn.hide()
        self.FiltKADSeries.ui.menuBtn.hide()
        
        view = self.FiltKADSeries.getView()
        view.setBackgroundColor(self.parent.color1)
        
        # Otsu1Series: Otsu n°1 classes definition
        self.Otsu1Series.ui.histogram.hide()
        self.Otsu1Series.ui.roiBtn.hide()
        self.Otsu1Series.ui.menuBtn.hide()
        
        view = self.Otsu1Series.getView()
        view.setBackgroundColor(self.parent.color1)
        
        # Binary1Series: Otsu n°1 after binarisation
        self.Binary1Series.ui.histogram.hide()
        self.Binary1Series.ui.roiBtn.hide()
        self.Binary1Series.ui.menuBtn.hide()
        
        view = self.Binary1Series.getView()
        view.setBackgroundColor(self.parent.color1)
        
        # Otsu2Series: Otsu n°2 classes definition
        self.Otsu2Series.ui.histogram.hide()
        self.Otsu2Series.ui.roiBtn.hide()
        self.Otsu2Series.ui.menuBtn.hide()
        
        view = self.Otsu2Series.getView()
        view.setBackgroundColor(self.parent.color1)
        
        # Binary2Series: Otsu n°2 after binarisation
        self.Binary2Series.ui.histogram.hide()
        self.Binary2Series.ui.roiBtn.hide()
        self.Binary2Series.ui.menuBtn.hide()
        
        view = self.Binary2Series.getView()
        view.setBackgroundColor(self.parent.color1)
        
    def mouseMoved(self, e):
        pos = e[0]
        sender = self.sender()
  
        if not self.mouseLock.isChecked():
            if self.KADSeries.view.sceneBoundingRect().contains(pos)\
                or self.FiltKADSeries.view.sceneBoundingRect().contains(pos)\
                or self.Otsu1Series.view.sceneBoundingRect().contains(pos)\
                or self.Binary1Series.view.sceneBoundingRect().contains(pos)\
                or self.Otsu2Series.view.sceneBoundingRect().contains(pos)\
                or self.Binary2Series.view.sceneBoundingRect().contains(pos):   
                
                if sender == self.proxy1:
                    item = self.KADSeries.view
                elif sender == self.proxy3:
                    item = self.FiltKADSeries.view
                elif sender == self.proxy5:
                    item = self.Otsu1Series.view
                elif sender == self.proxy7:
                    item = self.Binary1Series.view
                elif sender == self.proxy9:
                    item = self.Otsu2Series.view
                else:
                    item = self.Binary2Series.view
                
                mousePoint = item.mapSceneToView(pos) 
                     
                self.crosshair_v1.setPos(mousePoint.x())
                self.crosshair_h1.setPos(mousePoint.y())
                
                self.crosshair_v2.setPos(mousePoint.x())
                self.crosshair_h2.setPos(mousePoint.y())
                
                self.crosshair_v3.setPos(mousePoint.x())
                self.crosshair_h3.setPos(mousePoint.y())
                
                self.crosshair_v4.setPos(mousePoint.x())
                self.crosshair_h4.setPos(mousePoint.y())
                
                self.crosshair_v5.setPos(mousePoint.x())
                self.crosshair_h5.setPos(mousePoint.y())
                
                self.crosshair_v6.setPos(mousePoint.x())
                self.crosshair_h6.setPos(mousePoint.y())

            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
            self.printClick(self.x, self.y, sender)
    
    def mouseClick(self, e):
        pos = e[0]
        
        self.mouseLock.toggle()
        
        fromPosX = pos.scenePos()[0]
        fromPosY = pos.scenePos()[1]
        
        posQpoint = QtCore.QPointF()
        posQpoint.setX(fromPosX)
        posQpoint.setY(fromPosY)

        if self.KADSeries.view.sceneBoundingRect().contains(posQpoint):
                
            item = self.KADSeries.view
            mousePoint = item.mapSceneToView(posQpoint) 

            self.crosshair_v1.setPos(mousePoint.x())
            self.crosshair_h1.setPos(mousePoint.y())
                 
            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
    def printClick(self, x, y, sender):
        try: # Display values of Otsu1 and Otsu2
            self.Otsu1_label.setText("Otsu classes n°1: " + str(self.regions[x, y]))
            self.Otsu2_label.setText("Otsu classes n°2: " + str(self.regions_distance[x, y]))
        except:
            pass
            
        if self.flag_restored_only == True: # Display value of restored
            try:
                self.Threshold2_label.setText("Threshold n°2: " + str(self.restored_grains[x, y]))
            except:
                pass
            
        if self.flag_restored_struc == True: # Display value of restored (threshold)
            try:
                self.Threshold2_label.setText("Threshold n°2: " + str(self.binary_regions2[x, y]))        
            except:
                pass
        
        if self.flag_overlay == True: # Display value of KAD (with restored as 1)
            try:
                self.Threshold2_label.setText("Map overlay: " + str(np.round(self.overlay_KAD_restored[x, y],2)))        
            except:
                pass


# -*- coding: utf-8 -*-
"""
Created on Wed Nov 22 23:26:20 2023

@author: clanglois1
"""
import os
from os.path import abspath
from scipy import fftpack

from inspect import getsourcefile
import numpy as np

from pyqtgraph.Qt import QtGui, QtCore
import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/RemovedFFT_v2_TSG.ui")

class MainWindow(uiclass, baseclass):

    def __init__(self, parent):
        super().__init__()
        self.setupUi(self)
        self.parent = parent
        
        self.setWindowIcon(QtGui.QIcon('icons/background_icon.png'))
                
        self.expStack = parent.Current_stack
        self.filtered_Stack = np.copy(parent.Current_stack)
        
        self.img_number = len(self.expStack)
        self.stack_height = len(self.expStack[0])
        self.stack_width = len(self.expStack[0][0])
        self.img_step = int(360/len(self.expStack[:,]))

        self.x = 0
        self.y = 0
        
        self.fft = 1
        
        self.label_fft.setText("FFT Order: " + str(self.fft))
        
        if self.filtered.isChecked():
            self.filtered.toggle()
            
        self.filtered.setCheckable(False)

        self.crosshair_v1= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h1 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.plotIt = self.profiles.getPlotItem()
        self.plotIt.addLine(x = self.expSeries.currentIndex)
        
        self.proxy1 = pg.SignalProxy(self.expSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy4 = pg.SignalProxy(self.expSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)
        
        self.slider_fft.valueChanged.connect(self.fft_changed)
        self.expSeries.timeLine.sigPositionChanged.connect(self.drawCHORDprofiles)
        self.preview.clicked.connect(self.FFTStack)
        self.filtered.stateChanged.connect(self.drawCHORDprofiles)
        self.mouseLock.setVisible(False)
        
        self.Validate_button.clicked.connect(self.validate)
        
        self.prgbar = 0 # Outil pour la bar de progression
        self.progressBar.setValue(self.prgbar)
        self.progressBar.setRange(0, self.stack_height-1)

        self.displayExpStack(self.filtered_Stack)
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.15))
        self.resize(int(geometry.width() * 0.7), int(geometry.height() * 0.6))
        self.screen = screen
        
        self.defaultIV()
        self.defaultdrawCHORDprofiles()
        
        self.Validate_button.setEnabled(False) # Validation button is disables 

#%% Functions
    def fft_changed(self): # Extract FFT order to be considered
        self.fft = self.slider_fft.value()
        self.label_fft.setText("FFT Order: " + str(self.fft))

    def FFTStack(self): # Apply background remover
        self.filtered.setEnabled(False)
        self.preview.setEnabled(False)
        
        stack_reshape = np.moveaxis(self.expStack, 0, 2)

        stack_filtered = np.zeros((self.stack_height, self.stack_width, self.img_number))

        for x in range(self.stack_height):
            
            QApplication.processEvents()    
            self.ValSlice = x
            self.progression_bar()
            
            for y in range(self.stack_width):
                sig  = stack_reshape[x,y]
                sample_freq = fftpack.fftfreq(sig.size, d=self.img_step/360)
                sig_fft = fftpack.fft(sig)

                low_freq_fft = sig_fft.copy()
                high_freq_fft = sig_fft.copy()

                low_freq_fft[np.abs(sample_freq) <= self.fft] = 0
                high_freq_fft[np.abs(sample_freq) > self.fft] = 0

                signal_filtered = fftpack.ifft(low_freq_fft)
                stack_filtered[x,y] = np.real(signal_filtered)

        stack_filtered = np.moveaxis(stack_filtered, -1, 0)
        stack_filtered = np.asarray(stack_filtered, dtype=np.float32)
        
        self.filtered_Stack = stack_filtered
    
        self.displayExpStack(self.filtered_Stack)
        self.drawCHORDprofiles()                

        self.filtered.setEnabled(True)         
        self.filtered.setCheckable(True)
        self.filtered.setChecked(True)
        self.preview.setEnabled(True)
        self.Validate_button.setEnabled(True) # Validation button is enables 
        
    def validate(self):
        self.parent.Current_stack = np.copy(self.filtered_Stack)
        self.parent.StackList.append(self.filtered_Stack)

        Combo_text = '\u2022 Background substraction. FFT order : ' + str(self.fft)
        Combo_data = self.filtered_Stack
        self.parent.choiceBox.addItem(Combo_text, Combo_data)

        self.parent.displayExpStack(self.parent.Current_stack)
        
        self.parent.Info_box.ensureCursorVisible()
        self.parent.Info_box.insertPlainText("\n \u2022 Background substraction is achieved.")
        
        self.close()
   
    def drawCHORDprofiles(self):
        try:
            self.profiles.clear()
            line = self.plotIt.addLine(x = self.expSeries.currentIndex)
            line.setPen({'color': (42, 42, 42, 100), 'width': 2})
            
            self.legend = self.profiles.addLegend(horSpacing = 30, labelTextSize = '10pt', colCount = 1, labelTextColor = 'black', brush = self.parent.color6, pen = pg.mkPen(color=(0, 0, 0), width=1))
            
            pen = pg.mkPen(color=self.parent.color4, width=5) # Color and line width of the profile
            self.profiles.plot(self.expStack[:, self.x, self.y], pen=pen, name='Before FFT') # Plot of the profile
            
            styles = {"color": "black", "font-size": "40px", "font-family": "Noto Sans Cond"} # Style for labels
            self.profiles.setLabel("left", "Grayscale value", **styles) # Import style for Y label
            self.profiles.setLabel("bottom", "Slice", **styles) # Import style for X label
            
            font=QtGui.QFont('Noto Sans Cond', 9)
            
            self.profiles.getAxis("left").setTickFont(font) # Apply size of the ticks label
            self.profiles.getAxis("left").setStyle(tickTextOffset = 20) # Apply a slight offset
            self.profiles.getAxis("bottom").setTickFont(font) # Apply size of the ticks label
            self.profiles.getAxis("bottom").setStyle(tickTextOffset = 20) # Apply a slight offset
            
            self.profiles.getAxis('left').setTextPen('k') # Set the axis in black
            self.profiles.getAxis('bottom').setTextPen('k') # Set the axis in black
            
            self.profiles.setBackground(self.parent.color2)
            self.profiles.showGrid(x=True, y=True)
            
            if self.filtered.isChecked():
                pen2 = pg.mkPen(color=self.parent.color5, width=5) # Color and line width of the profile

                self.profiles.plot(self.filtered_Stack[:, self.x, self.y], pen=pen2, name='After FFT')
        except:
            pass

    def progression_bar(self): # Fonction relative à la barre de progression
        self.prgbar = self.ValSlice
        self.progressBar.setValue(self.prgbar)

    def displayExpStack(self, Series):
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        self.expSeries.addItem(self.crosshair_v1, ignoreBounds=True)
        self.expSeries.addItem(self.crosshair_h1, ignoreBounds=True) 
        
        view = self.expSeries.getView()
        state = view.getState()        
        self.expSeries.setImage(Series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.expSeries.timeLine.setPen(color=self.parent.color3, width=15)
        self.expSeries.frameTicks.setPen(color=self.parent.color1, width=5)
        self.expSeries.frameTicks.setYRange((0, 1))

        s = self.expSeries.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 

    def defaultIV(self):
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        view = self.expSeries.getView()
        view.setBackgroundColor(self.parent.color1)
        
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
    
    def defaultdrawCHORDprofiles(self):
        self.profiles.clear()
        self.profiles.setBackground(self.parent.color2)
        
        self.profiles.getPlotItem().hideAxis('bottom')
        self.profiles.getPlotItem().hideAxis('left')

    def mouseMoved(self, e):
        pos = e[0]

        if not self.mouseLock.isChecked():
            if self.expSeries.view.sceneBoundingRect().contains(pos):
    
                item = self.expSeries.view
                mousePoint = item.mapSceneToView(pos) 
                     
                self.crosshair_v1.setPos(mousePoint.x())
                self.crosshair_h1.setPos(mousePoint.y())

            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
            if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0]) and self.y < len(self.expStack[0, 0, :]):
                self.drawCHORDprofiles()

    def mouseClick(self, e):
        pos = e[0]
        
        self.mouseLock.toggle()
        
        fromPosX = pos.scenePos()[0]
        fromPosY = pos.scenePos()[1]
        
        posQpoint = QtCore.QPointF()
        posQpoint.setX(fromPosX)
        posQpoint.setY(fromPosY)

        if self.expSeries.view.sceneBoundingRect().contains(posQpoint):
                
            item = self.expSeries.view
            mousePoint = item.mapSceneToView(posQpoint) 

            self.crosshair_v1.setPos(mousePoint.x())
            self.crosshair_h1.setPos(mousePoint.y())
                 
            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
        if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0])and self.y < len(self.expStack[0, 0, :]):
            self.drawCHORDprofiles()


# -*- coding: utf-8 -*-
"""
Created on Sun Nov 26 17:31:45 2023

@author: clanglois1
"""

import os
from os.path import abspath

from inspect import getsourcefile
import numpy as np

from pyqtgraph.Qt import QtGui, QtCore
import pyqtgraph as pg
from PyQt5.QtWidgets import QApplication

from inichord import general_functions as gf

path2thisFile = abspath(getsourcefile(lambda:0))
uiclass, baseclass = pg.Qt.loadUiType(os.path.dirname(path2thisFile) + "/VSNR_v2_TSG.ui")

class MainWindow(uiclass, baseclass):
    def __init__(self, parent):
        super().__init__()
        self.setupUi(self)
        self.parent = parent
        
        self.setWindowIcon(QtGui.QIcon('icons/filter_icon.png'))   
        
        self.expStack = parent.Current_stack
        self.denoised_Stack = np.copy(parent.Current_stack)

        self.x = 0
        self.y = 0
        
        self.noise = self.slider_noise.value() / 100.0
        self.label_noise.setText("Noise level: " + str(self.noise))
        
        if self.denoised.isChecked():
            self.denoised.toggle()
            
        self.denoised.setCheckable(False)

        self.crosshair_v1= pg.InfiniteLine(angle=90, movable=False, pen=self.parent.color5)
        self.crosshair_h1 = pg.InfiniteLine(angle=0, movable=False, pen=self.parent.color5)
        
        self.plotIt = self.profiles.getPlotItem()
        self.plotIt.addLine(x = self.expSeries.currentIndex)
        
        self.proxy1 = pg.SignalProxy(self.expSeries.scene.sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.proxy4 = pg.SignalProxy(self.expSeries.ui.graphicsView.scene().sigMouseClicked, rateLimit=60, slot=self.mouseClick)
        
        self.slider_noise.sliderReleased.connect(self.noise_changed)
       
        self.expSeries.timeLine.sigPositionChanged.connect(self.drawCHORDprofiles)
        self.preview.clicked.connect(self.denoiseStack)
        self.denoised.stateChanged.connect(self.drawCHORDprofiles)

        self.Validate_button.clicked.connect(self.validate)
        self.mouseLock.setVisible(False)
        
        self.img_number = len(self.expStack)
        self.prgbar = 0 # Outil pour la bar de progression
        self.progressBar.setValue(self.prgbar)
        self.progressBar.setRange(0, self.img_number-1)
        
        self.displayExpStack(self.expStack)
        self.defaultdrawCHORDprofiles()
        
        app = QApplication.instance()
        screen = app.screenAt(self.pos())
        geometry = screen.availableGeometry()
        
        self.move(int(geometry.width() * 0.1), int(geometry.height() * 0.15))
        self.resize(int(geometry.width() * 0.7), int(geometry.height() * 0.6))
        self.screen = screen
        
        self.Validate_button.setEnabled(True)

#%% Functions
    def noise_changed(self):
        self.noise = self.slider_noise.value() / 100.0            
        self.label_noise.setText("Noise level: " + str(self.noise))
        self.denoiseSlice()
    
    def denoiseSlice(self):
        filter_ = [{'name':'Dirac', 'noise_level':self.noise}]  

        self.denoised_Stack[0, :, :] = gf.VSNR_funct(self.expStack[0, :, :], filter_)
        self.displayExpStack(self.denoised_Stack)
        self.denoised.setEnabled(False)

    def denoiseStack(self):
        self.denoised.setEnabled(False)
        self.preview.setEnabled(False)
        
        filter_ = [{'name':'Dirac', 'noise_level':self.noise}] 
        
        for i in range(0,len(self.expStack[:, 0, 0])): # Apply parameter on each slice
            self.denoised_Stack[i, :, :] = gf.VSNR_funct(self.expStack[i, :, :], filter_)

            QApplication.processEvents()    
            self.ValSlice = i
            self.progression_bar()
                
        self.denoised.setCheckable(True)
        self.denoised.setEnabled(True)  
        self.denoised.setChecked(True)
        self.preview.setEnabled(True)
        self.Validate_button.setEnabled(True)
        
        self.displayExpStack(self.denoised_Stack)
        
    def validate(self):
        self.parent.Current_stack = np.copy(self.denoised_Stack)
        self.parent.StackList.append(self.denoised_Stack)
        
        Combo_text = '\u2022 Manual VSNR denoising. Alpha parameter : ' + str(self.noise) + '. Mode : Dirac'
        Combo_data = self.denoised_Stack
        self.parent.choiceBox.addItem(Combo_text, Combo_data)

        self.parent.displayExpStack(self.parent.Current_stack)

        self.drawCHORDprofiles()        
        
        self.parent.Info_box.ensureCursorVisible()
        self.parent.Info_box.insertPlainText("\n \u2022 VSNR denoising is achieved.")
        
        self.close()

    def defaultdrawCHORDprofiles(self):
        self.profiles.clear()
        self.profiles.setBackground(self.parent.color2)
        
        self.profiles.getPlotItem().hideAxis('bottom')
        self.profiles.getPlotItem().hideAxis('left')
         
    def drawCHORDprofiles(self):
        try:
            self.profiles.clear()
            line = self.plotIt.addLine(x = self.expSeries.currentIndex)
            line.setPen({'color': (42, 42, 42, 100), 'width': 2})
            
            self.legend = self.profiles.addLegend(horSpacing = 30, labelTextSize = '10pt', colCount = 1, labelTextColor = 'black', brush = self.parent.color6, pen = pg.mkPen(color=(0, 0, 0), width=1))
            
            pen = pg.mkPen(color=self.parent.color4, width=5) # Color and line width of the profile
            self.profiles.plot(self.expStack[:, self.x, self.y], pen=pen, name='Undenoised') # Plot of the profile
            
            styles = {"color": "black", "font-size": "40px", "font-family": "Noto Sans Cond"} # Style for labels
            self.profiles.setLabel("left", "Grayscale value", **styles) # Import style for Y label
            self.profiles.setLabel("bottom", "Slice", **styles) # Import style for X label
            
            font=QtGui.QFont('Noto Sans Cond', 9)
            
            self.profiles.getAxis("left").setTickFont(font) # Apply size of the ticks label
            self.profiles.getAxis("left").setStyle(tickTextOffset = 20) # Apply a slight offset
            self.profiles.getAxis("bottom").setTickFont(font) # Apply size of the ticks label
            self.profiles.getAxis("bottom").setStyle(tickTextOffset = 20) # Apply a slight offset
            
            self.profiles.getAxis('left').setTextPen('k') # Set the axis in black
            self.profiles.getAxis('bottom').setTextPen('k') # Set the axis in black
            
            self.profiles.setBackground(self.parent.color2)
            self.profiles.showGrid(x=True, y=True)
            
            if self.denoised.isChecked():
                pen2 = pg.mkPen(color=self.parent.color5, width=5) # Color and line width of the profile

                self.profiles.plot(self.denoised_Stack[:, self.x, self.y], pen=pen2, name='Denoised')
        except:
            pass

    def progression_bar(self): # Fonction relative à la barre de progression
        self.prgbar = self.ValSlice
        self.progressBar.setValue(self.prgbar)
        
    def displayExpStack(self, Series):
        self.expSeries.ui.histogram.hide()
        self.expSeries.ui.roiBtn.hide()
        self.expSeries.ui.menuBtn.hide()
        
        self.expSeries.addItem(self.crosshair_v1, ignoreBounds=True)
        self.expSeries.addItem(self.crosshair_h1, ignoreBounds=True) 
        
        view = self.expSeries.getView()
        state = view.getState()        
        self.expSeries.setImage(Series) 
        view.setState(state)
        
        view.setBackgroundColor(self.parent.color1)
        ROIplot = self.expSeries.getRoiPlot()
        ROIplot.setBackground(self.parent.color1)
        
        font=QtGui.QFont('Noto Sans Cond', 9)
        ROIplot.getAxis("bottom").setTextPen('k') # Apply size of the ticks label
        ROIplot.getAxis("bottom").setTickFont(font)
        
        self.expSeries.timeLine.setPen(color=self.parent.color3, width=15)
        self.expSeries.frameTicks.setPen(color=self.parent.color1, width=5)
        self.expSeries.frameTicks.setYRange((0, 1))

        s = self.expSeries.ui.splitter
        s.handle(1).setEnabled(True)
        s.setStyleSheet("background: 5px white;")
        s.setHandleWidth(5) 
        
    def mouseMoved(self, e):
        pos = e[0]

        if not self.mouseLock.isChecked():
            if self.expSeries.view.sceneBoundingRect().contains(pos):
    
                item = self.expSeries.view
                mousePoint = item.mapSceneToView(pos) 
                     
                self.crosshair_v1.setPos(mousePoint.x())
                self.crosshair_h1.setPos(mousePoint.y())

            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())
            
            if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0]) and self.y < len(self.expStack[0, 0, :]):
                self.drawCHORDprofiles()

    def mouseClick(self, e):
        pos = e[0]
        
        self.mouseLock.toggle()
        
        fromPosX = pos.scenePos()[0]
        fromPosY = pos.scenePos()[1]
        
        posQpoint = QtCore.QPointF()
        posQpoint.setX(fromPosX)
        posQpoint.setY(fromPosY)

        if self.expSeries.view.sceneBoundingRect().contains(posQpoint):

            item = self.expSeries.view
            mousePoint = item.mapSceneToView(posQpoint) 

            self.crosshair_v1.setPos(mousePoint.x())
            self.crosshair_h1.setPos(mousePoint.y())
 
            self.x = int(mousePoint.x())
            self.y = int(mousePoint.y())

        if self.x >= 0 and self.y >= 0 and self.x < len(self.expStack[0, :, 0])and self.y < len(self.expStack[0, 0, :]):
            self.drawCHORDprofiles()